# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Utilities Cloud GKE Multi-cloud for Azure API."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from apitools.base.py import list_pager
from googlecloudsdk.api_lib.container.gkemulticloud import util
from googlecloudsdk.calliope import base


class _AzureClientBase(object):
  """Base class for Azure clients."""

  def __init__(self, client=None, messages=None, track=base.ReleaseTrack.GA):
    self.track = track
    self.client = client or util.GetClientInstance(track)
    self.messages = messages or util.GetMessagesModule(track)
    self._service = self._GetService()
    self.version = util.GetApiVersionForTrack(track).capitalize()

  def List(self, parent_ref, page_size, limit):
    """List Azure resources."""

    req = self._service.GetRequestType('List')(parent=parent_ref.RelativeName())

    for item in list_pager.YieldFromList(
        self._service,
        req,
        field=self.GetListResultsField(),
        limit=limit,
        batch_size=page_size,
        batch_size_attribute='pageSize'):
      yield item

  def Get(self, resource_ref):
    """Get an Azure resource."""

    req = self._service.GetRequestType('Get')(name=resource_ref.RelativeName())
    return self._service.Get(req)

  def Delete(self, resource_ref, validate_only=None, allow_missing=None):
    """Delete an Azure resource."""

    req = self._service.GetRequestType('Delete')(
        name=resource_ref.RelativeName())
    if validate_only:
      req.validateOnly = True
    if allow_missing:
      req.allowMissing = True

    return self._service.Delete(req)

  def _GetService(self):
    raise NotImplementedError(
        '_GetService() method not implemented for this type')

  def GetListResultsField(self):
    raise NotImplementedError(
        'GetListResultsField() method not implemented for this type')

  def _CreateAzureDiskTemplate(self, size_gib):
    msg = 'GoogleCloudGkemulticloud{}AzureDiskTemplate'.format(self.version)
    return getattr(self.messages, msg)(sizeGib=size_gib)

  def _CreateProxyConfig(self, **kwargs):
    msg = 'GoogleCloudGkemulticloud{}AzureProxyConfig'.format(self.version)
    return getattr(self.messages, msg)(**kwargs)

  def _CreateConfigEncryption(self, **kwargs):
    msg = 'GoogleCloudGkemulticloud{}AzureConfigEncryption'.format(self.version)
    return getattr(self.messages, msg)(**kwargs)


class ClustersClient(_AzureClientBase):
  """Client for Azure Clusters in the gkemulticloud API."""

  def Create(self,
             cluster_ref,
             client_ref=None,
             azure_region=None,
             resource_group_id=None,
             vnet_id=None,
             pod_address_cidr_blocks=None,
             service_address_cidr_blocks=None,
             cluster_version=None,
             subnet_id=None,
             vm_size=None,
             ssh_public_key=None,
             proxy_resource_group_id=None,
             proxy_secret_id=None,
             main_volume_size=None,
             root_volume_size=None,
             validate_only=False,
             tags=None,
             admin_users=None,
             replica_placements=None,
             fleet_project=None,
             service_load_balancer_subnet_id=None,
             endpoint_subnet_id=None,
             database_encryption_key_id=None,
             config_encryption_key_id=None,
             config_encryption_public_key=None,
             logging=None):
    """Create a new Azure Cluster."""
    req = self._service.GetRequestType('Create')(
        azureClusterId=cluster_ref.azureClustersId,
        parent=cluster_ref.Parent().RelativeName())
    if validate_only:
      req.validateOnly = True

    c = self._AddAzureCluster(req)
    c.name = cluster_ref.azureClustersId
    c.azureClient = client_ref.RelativeName()
    c.azureRegion = azure_region
    c.resourceGroupId = resource_group_id
    c.authorization = self._CreateAuthorization(admin_users)

    net = self._AddAzureNetworking(c)
    net.podAddressCidrBlocks.append(pod_address_cidr_blocks)
    net.serviceAddressCidrBlocks.append(service_address_cidr_blocks)
    net.virtualNetworkId = vnet_id
    if service_load_balancer_subnet_id is not None:
      net.serviceLoadBalancerSubnetId = service_load_balancer_subnet_id

    cp = self._AddAzureControlPlane(c)

    if main_volume_size:
      cp.mainVolume = self._CreateAzureDiskTemplate(main_volume_size)
    if root_volume_size:
      cp.rootVolume = self._CreateAzureDiskTemplate(root_volume_size)

    cp.sshConfig = self._CreateSshConfig(authorizedKey=ssh_public_key)
    cp.subnetId = subnet_id
    cp.version = cluster_version
    cp.vmSize = vm_size

    if proxy_resource_group_id is not None and proxy_secret_id is not None:
      cp.proxyConfig = self._CreateProxyConfig(
          resourceGroupId=proxy_resource_group_id, secretId=proxy_secret_id)

    if replica_placements:
      cp.replicaPlacements.extend(replica_placements)

    if endpoint_subnet_id is not None:
      cp.endpointSubnetId = endpoint_subnet_id

    if database_encryption_key_id is not None:
      cp.databaseEncryption = self._CreateDatabaseEncryption(
          keyId=database_encryption_key_id)

    if config_encryption_key_id is not None:
      cp.configEncryption = self._CreateConfigEncryption(
          keyId=config_encryption_key_id,
          publicKey=config_encryption_public_key)

    if tags:
      tag_type = type(cp).TagsValue.AdditionalProperty
      cp.tags = type(cp).TagsValue(additionalProperties=[
          tag_type(key=k, value=v) for k, v in tags.items()
      ])

    if fleet_project:
      c.fleet = self._CreateFleet(fleet_project)

    if logging:
      c.loggingConfig = logging

    return self._service.Create(req)

  def GenerateAccessToken(self, cluster_ref):
    """Generates an access token for an Azure cluster."""
    req = self._service.GetRequestType('GenerateAzureAccessToken')(
        azureCluster=cluster_ref.RelativeName())
    return self._service.GenerateAzureAccessToken(req)

  def GetListResultsField(self):
    return 'azureClusters'

  def Update(self,
             cluster_ref,
             client_ref=None,
             cluster_version=None,
             vm_size=None,
             admin_users=None,
             validate_only=False):
    """Updates an Anthos cluster on Azure.

    Args:
      cluster_ref: obj, Cluster object to be updated.
      client_ref: obj, Client to use in the cluster update.
      cluster_version: str, Cluster version to use in the cluster update.
      vm_size: str, control plane VM size to use in the cluster update.
      admin_users: list, Users that can act as a cluster administrator.
      validate_only: bool, Validate the update of the cluster.

    Returns:
      Response to the update request.
    """
    req = self._service.GetRequestType('Patch')(
        name=cluster_ref.RelativeName(),)
    if validate_only:
      req.validateOnly = True

    update_mask = []

    c = self._AddAzureCluster(req)
    if client_ref:
      c.azureClient = client_ref.RelativeName()
      update_mask.append('azure_client')

    cp = self._AddAzureControlPlane(c)
    if cluster_version:
      cp.version = cluster_version
      update_mask.append('control_plane.version')
    if vm_size:
      cp.vmSize = vm_size
      update_mask.append('control_plane.vm_size')

    if admin_users:
      c.authorization = self._CreateAuthorization(admin_users)
      update_mask.append('authorization.admin_users')

    req.updateMask = ','.join(update_mask)

    return self._service.Patch(req)

  def _GetService(self):
    return self.client.projects_locations_azureClusters

  def _AddAzureCluster(self, req):
    # Using this to hide the 'v1alpha' that shows up in the type.
    version = util.GetApiVersionForTrack(self.track).capitalize()
    msg = 'GoogleCloudGkemulticloud{}AzureCluster'.format(version)
    attr = 'googleCloudGkemulticloud{}AzureCluster'.format(version)
    cluster = getattr(self.messages, msg)()
    setattr(req, attr, cluster)
    return cluster

  def _AddAzureNetworking(self, req):
    # Using this to hide the 'v1alpha' that shows up in the type.
    version = util.GetApiVersionForTrack(self.track).capitalize()
    msg = 'GoogleCloudGkemulticloud{}AzureClusterNetworking'.format(version)
    net = getattr(self.messages, msg)()
    req.networking = net
    return net

  def _AddAzureControlPlane(self, req):
    # Using this to hide the 'v1alpha' that shows up in the type.
    version = util.GetApiVersionForTrack(self.track).capitalize()
    msg = 'GoogleCloudGkemulticloud{}AzureControlPlane'.format(version)
    cp = getattr(self.messages, msg)()
    req.controlPlane = cp
    return cp

  def _CreateDatabaseEncryption(self, **kwargs):
    version = util.GetApiVersionForTrack(self.track).capitalize()
    msg = 'GoogleCloudGkemulticloud{}AzureDatabaseEncryption'.format(version)
    return getattr(self.messages, msg)(**kwargs)

  def _CreateFleet(self, fleet_project):
    version = util.GetApiVersionForTrack(self.track).capitalize()
    msg = 'GoogleCloudGkemulticloud{}Fleet'.format(version)
    fleet = getattr(self.messages, msg)(project=fleet_project)
    return fleet

  def _CreateSshConfig(self, **kwargs):
    # Using this to hide the 'v1alpha' that shows up in the type.
    version = util.GetApiVersionForTrack(self.track).capitalize()
    msg = 'GoogleCloudGkemulticloud{}AzureSshConfig'.format(version)
    return getattr(self.messages, msg)(**kwargs)

  def _CreateAuthorization(self, admin_users):
    """Create an Azure Authorization message.

    Args:
      admin_users: list of strings, admin users.

    Returns:
      A GoogleCloudGkemulticloudAzureAuthorization message.
    """
    # Using this to hide the 'v1alpha' that shows up in the type.
    version = util.GetApiVersionForTrack(self.track).capitalize()

    cluster_user_msg = 'GoogleCloudGkemulticloud{}AzureClusterUser'.format(
        version)

    users = []
    if admin_users:
      users = [
          getattr(self.messages, cluster_user_msg)(username=u)
          for u in admin_users
      ]

    msg = 'GoogleCloudGkemulticloud{}AzureAuthorization'.format(version)
    return getattr(self.messages, msg)(adminUsers=users)


class NodePoolsClient(_AzureClientBase):
  """Client for Azure Node Pools in the gkemulticloud API."""

  def __init__(self, **kwargs):
    self._message_prefix = 'GoogleCloudGkemulticloud'
    self.__attribute_prefix = 'googleCloudGkemulticloud'
    super(NodePoolsClient, self).__init__(**kwargs)

  def __AddMessageForTrack(self, req, type_name, **kwargs):
    """For adding messages where attribute and type have a very long name."""
    version = util.GetApiVersionForTrack(self.track).capitalize()
    msg = '{}{}{}'.format(self._message_prefix, version, type_name)
    attr = '{}{}{}'.format(self.__attribute_prefix, version, type_name)
    instance = getattr(self.messages, msg)(**kwargs)
    setattr(req, attr, instance)
    return instance

  def Create(self,
             nodepool_ref,
             node_version=None,
             subnet_id=None,
             vm_size=None,
             ssh_public_key=None,
             proxy_resource_group_id=None,
             proxy_secret_id=None,
             root_volume_size=None,
             tags=None,
             validate_only=None,
             min_nodes=None,
             max_nodes=None,
             max_pods_per_node=None,
             taints=None,
             labels=None,
             azure_availability_zone=None,
             config_encryption_key_id=None,
             config_encryption_public_key=None,
             image_type=None):
    """Create a new Azure Node Pool."""
    req = self._service.GetRequestType('Create')(
        azureNodePoolId=nodepool_ref.azureNodePoolsId,
        parent=nodepool_ref.Parent().RelativeName(),
        validateOnly=validate_only)

    nodepool = self.__AddMessageForTrack(
        req,
        'AzureNodePool',
        name=nodepool_ref.azureNodePoolsId,
        subnetId=subnet_id,
        version=node_version)
    nodepool.azureAvailabilityZone = azure_availability_zone

    nodepool.autoscaling = type(nodepool).autoscaling.type(
        maxNodeCount=max_nodes, minNodeCount=min_nodes)

    nodepool.maxPodsConstraint = type(nodepool).maxPodsConstraint.type(
        maxPodsPerNode=max_pods_per_node)

    nodepool.config = type(nodepool).config.type(vmSize=vm_size)

    nodeconfig = nodepool.config
    nodeconfig.sshConfig = type(nodeconfig).sshConfig.type(
        authorizedKey=ssh_public_key)
    nodeconfig.taints.extend(taints)

    if config_encryption_key_id is not None:
      nodeconfig.configEncryption = self._CreateConfigEncryption(
          keyId=config_encryption_key_id,
          publicKey=config_encryption_public_key)

    if proxy_resource_group_id is not None and proxy_secret_id is not None:
      nodeconfig.proxyConfig = self._CreateProxyConfig(
          resourceGroupId=proxy_resource_group_id, secretId=proxy_secret_id)

    if root_volume_size:
      nodeconfig.rootVolume = self._CreateAzureDiskTemplate(root_volume_size)

    if tags:
      tag_type = type(nodeconfig).TagsValue.AdditionalProperty
      nodeconfig.tags = type(nodeconfig).TagsValue(additionalProperties=[
          tag_type(key=k, value=v) for k, v in tags.items()
      ])

    if labels:
      label_type = type(nodeconfig).LabelsValue.AdditionalProperty
      nodeconfig.labels = type(nodeconfig).LabelsValue(additionalProperties=[
          label_type(key=k, value=v) for k, v in labels.items()
      ])

    if image_type:
      nodeconfig.imageType = image_type

    return self._service.Create(req)

  def Update(self,
             nodepool_ref,
             node_version=None,
             min_nodes=None,
             max_nodes=None,
             validate_only=None):
    """Updates a node pool in an Anthos cluster on Azure including node pool version.

    Args:
      nodepool_ref: obj, Node pool object to be updated.
      node_version: str, Node pool version to use in the node pool update.
      min_nodes: int, Minimum number of nodes in the node pool.
      max_nodes: int, Maximum number of nodes in the node pool.
      validate_only: bool, Validate the update of the node pool.

    Returns:
      Response to the update request.
    """
    req = self._service.GetRequestType('Patch')(
        name=nodepool_ref.RelativeName(),
        validateOnly=validate_only)

    update_mask = []

    nodepool = self.__AddMessageForTrack(
        req, 'AzureNodePool', name=nodepool_ref.azureNodePoolsId)

    if node_version:
      nodepool.version = node_version
      update_mask.append('version')

    nodepool.autoscaling = type(nodepool).autoscaling.type()
    if min_nodes is not None:
      nodepool.autoscaling.minNodeCount = min_nodes
      update_mask.append('autoscaling.minNodeCount')
    if max_nodes is not None:
      nodepool.autoscaling.maxNodeCount = max_nodes
      update_mask.append('autoscaling.maxNodeCount')

    req.updateMask = ','.join(update_mask)
    return self._service.Patch(req)

  def _GetService(self):
    return self.client.projects_locations_azureClusters_azureNodePools

  def GetListResultsField(self):
    return 'azureNodePools'


class ClientsClient(_AzureClientBase):
  """Client for Azure Clients in the gkemulticloud API."""

  def Create(self, client_ref, tenant_id, application_id, validate_only=False):
    """Create a new Azure client."""
    req = self.messages.GkemulticloudProjectsLocationsAzureClientsCreateRequest(
        azureClientId=client_ref.azureClientsId,
        parent=client_ref.Parent().RelativeName())
    if validate_only:
      req.validateOnly = True

    client = self._AddClient(req)
    client.name = client_ref.azureClientsId
    client.applicationId = application_id
    client.tenantId = tenant_id

    return self._service.Create(req)

  def GetListResultsField(self):
    return 'azureClients'

  def _GetService(self):
    return self.client.projects_locations_azureClients

  def _AddClient(self, req):
    # Using this to hide the 'v1alpha' that shows up in the type.
    version = util.GetApiVersionForTrack(self.track).capitalize()
    msg = 'GoogleCloudGkemulticloud{}AzureClient'.format(version)
    attr = 'googleCloudGkemulticloud{}AzureClient'.format(version)
    client = getattr(self.messages, msg)()
    setattr(req, attr, client)
    return client



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Remove IAM Policy Binding."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.iam import iam_util
from googlecloudsdk.command_lib.iap import util as iap_util


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class RemoveIamPolicyBinding(base.Command):
  """Remove IAM policy binding from an IAP gateway resource.

  Removes a policy binding from the IAM policy of an IAP gateway resource. One
  binding consists of a member, a role and an optional condition.
  """
  detailed_help = {
      'EXAMPLES':
          """\

          To remove an IAM policy binding for the role of 'roles/editor' for the
          user 'test-user@gmail.com', run:

            $ {command} --member='user:test-user@gmail.com' --role='roles/editor'

          To remove an IAM policy binding for the role of 'roles/editor' from
          all authenticated users,run:

            $ {command} --member='allAuthenticatedUsers' --role='roles/editor'

          To remove an IAM policy binding which expires at the end of the year
          2018 for the role of 'roles/browser' for the user 'test-user@gmail.com',
          run:

            $ {command} --member='user:test-user@gmail.com' --role='roles/browser'
                --condition='expression=request.time <
                timestamp("2019-01-01T00:00:00Z"),title=expires_end_of_2018,
                description=Expires at midnight on 2018-12-31'

          To remove all IAM policy bindings regardless of the condition for the
          role of 'roles/browser' and for the user 'test-user@gmail.com', run:

            $ {command} --member='user:test-user@gmail.com' --role='roles/browser'
            --all

          See https://cloud.google.com/iam/docs/managing-policies for details of
          policy role and member types.
  """,
  }

  @staticmethod
  def Args(parser):
    """Register flags for this command.

    Args:
      parser: An argparse.ArgumentParser-like object. It is mocked out in order
        to capture some information, but behaves like an ArgumentParser.
    """
    iap_util.AddRemoveIamPolicyBindingArgs(parser)
    base.URI_FLAG.RemoveFromParser(parser)

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Returns:
      The specified function with its description and configured filter.
    """
    condition = iam_util.ValidateAndExtractCondition(args)
    iap_iam_ref = iap_util.ParseIapGatewayResource(self.ReleaseTrack())
    return iap_iam_ref.RemoveIamPolicyBinding(args.member, args.role, condition,
                                              args.all)



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import AsyncIterator, Mapping, Optional

from google.cloud.pubsub_v1.types import BatchSettings

from google.cloud.pubsublite.admin_client import AdminClient
from google.cloud.pubsublite.internal.endpoints import regional_endpoint
from google.cloud.pubsublite.internal.wire.client_cache import ClientCache
from google.cloud.pubsublite.internal.wire.default_routing_policy import (
    DefaultRoutingPolicy,
)
from google.cloud.pubsublite.internal.wire.gapic_connection import (
    GapicConnectionFactory,
)
from google.cloud.pubsublite.internal.wire.merge_metadata import merge_metadata
from google.cloud.pubsublite.internal.wire.partition_count_watcher_impl import (
    PartitionCountWatcherImpl,
)
from google.cloud.pubsublite.internal.wire.partition_count_watching_publisher import (
    PartitionCountWatchingPublisher,
)
from google.cloud.pubsublite.internal.wire.publisher import Publisher
from google.cloud.pubsublite.internal.wire.single_partition_publisher import (
    SinglePartitionPublisher,
)
from google.cloud.pubsublite.types import Partition, TopicPath
from google.cloud.pubsublite.internal.routing_metadata import topic_routing_metadata
from google.cloud.pubsublite_v1 import InitialPublishRequest, PublishRequest
from google.cloud.pubsublite_v1.services.publisher_service import async_client
from google.api_core.client_options import ClientOptions
from google.auth.credentials import Credentials

DEFAULT_BATCHING_SETTINGS = BatchSettings(
    max_bytes=(
        3 * 1024 * 1024
    ),  # 3 MiB to stay 1 MiB below GRPC's 4 MiB per-message limit.
    max_messages=1000,
    max_latency=0.05,  # 50 ms
)
DEFAULT_PARTITION_POLL_PERIOD = 600  # ten minutes


def make_publisher(
    topic: TopicPath,
    transport: str,
    per_partition_batching_settings: Optional[BatchSettings] = None,
    credentials: Optional[Credentials] = None,
    client_options: Optional[ClientOptions] = None,
    metadata: Optional[Mapping[str, str]] = None,
) -> Publisher:
    """
    Make a new publisher for the given topic.

    Args:
      topic: The topic to publish to.
      transport: The transport type to use.
      per_partition_batching_settings: Settings for batching messages on each partition. The default is reasonable for most cases.
      credentials: The credentials to use to connect. GOOGLE_DEFAULT_CREDENTIALS is used if None.
      client_options: Other options to pass to the client. Note that if you pass any you must set api_endpoint.
      metadata: Additional metadata to send with the RPC.

    Returns:
      A new Publisher.

    Throws:
      GoogleApiCallException on any error determining topic structure.
    """
    if per_partition_batching_settings is None:
        per_partition_batching_settings = DEFAULT_BATCHING_SETTINGS
    admin_client = AdminClient(
        region=topic.location.region,
        credentials=credentials,
        client_options=client_options,
    )
    if client_options is None:
        client_options = ClientOptions(
            api_endpoint=regional_endpoint(topic.location.region)
        )
    client_cache = ClientCache(
        lambda: async_client.PublisherServiceAsyncClient(
            credentials=credentials, transport=transport, client_options=client_options
        )
    )

    def publisher_factory(partition: Partition):
        def connection_factory(requests: AsyncIterator[PublishRequest]):
            final_metadata = merge_metadata(
                metadata, topic_routing_metadata(topic, partition)
            )
            return client_cache.get().publish(
                requests, metadata=list(final_metadata.items())
            )

        return SinglePartitionPublisher(
            InitialPublishRequest(topic=str(topic), partition=partition.value),
            per_partition_batching_settings,
            GapicConnectionFactory(connection_factory),
        )

    def policy_factory(partition_count: int):
        return DefaultRoutingPolicy(partition_count)

    return PartitionCountWatchingPublisher(
        PartitionCountWatcherImpl(admin_client, topic, DEFAULT_PARTITION_POLL_PERIOD),
        publisher_factory,
        policy_factory,
    )



"""Generated message classes for datapipelines version v1.

Data Pipelines provides an interface for creating, updating, and managing
recurring Data Analytics jobs.
"""
# NOTE: This file is autogenerated and should not be edited by hand.

from __future__ import absolute_import

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding
from apitools.base.py import extra_types


package = 'datapipelines'


class DatapipelinesProjectsLocationsListPipelinesRequest(_messages.Message):
  r"""A DatapipelinesProjectsLocationsListPipelinesRequest object.

  Fields:
    filter: An expression for filtering the results of the request. If
      unspecified, all pipelines will be returned. Multiple filters can be
      applied and must be comma separated. Fields eligible for filtering are:
      + `type`: The type of the pipeline (streaming or batch). Allowed values
      are `ALL`, `BATCH`, and `STREAMING`. + `status`: The activity status of
      the pipeline. Allowed values are `ALL`, `ACTIVE`, `ARCHIVED`, and
      `PAUSED`. For example, to limit results to active batch processing
      pipelines: type:BATCH,status:ACTIVE
    pageSize: The maximum number of entities to return. The service may return
      fewer than this value, even if there are additional pages. If
      unspecified, the max limit is yet to be determined by the backend
      implementation.
    pageToken: A page token, received from a previous `ListPipelines` call.
      Provide this to retrieve the subsequent page. When paginating, all other
      parameters provided to `ListPipelines` must match the call that provided
      the page token.
    parent: Required. The location name. For example:
      `projects/PROJECT_ID/locations/LOCATION_ID`.
  """

  filter = _messages.StringField(1)
  pageSize = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(3)
  parent = _messages.StringField(4, required=True)


class DatapipelinesProjectsLocationsPipelinesCreateRequest(_messages.Message):
  r"""A DatapipelinesProjectsLocationsPipelinesCreateRequest object.

  Fields:
    googleCloudDatapipelinesV1Pipeline: A GoogleCloudDatapipelinesV1Pipeline
      resource to be passed as the request body.
    parent: Required. The location name. For example:
      `projects/PROJECT_ID/locations/LOCATION_ID`.
  """

  googleCloudDatapipelinesV1Pipeline = _messages.MessageField('GoogleCloudDatapipelinesV1Pipeline', 1)
  parent = _messages.StringField(2, required=True)


class DatapipelinesProjectsLocationsPipelinesDeleteRequest(_messages.Message):
  r"""A DatapipelinesProjectsLocationsPipelinesDeleteRequest object.

  Fields:
    name: Required. The pipeline name. For example:
      `projects/PROJECT_ID/locations/LOCATION_ID/pipelines/PIPELINE_ID`.
  """

  name = _messages.StringField(1, required=True)


class DatapipelinesProjectsLocationsPipelinesGetRequest(_messages.Message):
  r"""A DatapipelinesProjectsLocationsPipelinesGetRequest object.

  Fields:
    name: Required. The pipeline name. For example:
      `projects/PROJECT_ID/locations/LOCATION_ID/pipelines/PIPELINE_ID`.
  """

  name = _messages.StringField(1, required=True)


class DatapipelinesProjectsLocationsPipelinesJobsListRequest(_messages.Message):
  r"""A DatapipelinesProjectsLocationsPipelinesJobsListRequest object.

  Fields:
    pageSize: The maximum number of entities to return. The service may return
      fewer than this value, even if there are additional pages. If
      unspecified, the max limit will be determined by the backend
      implementation.
    pageToken: A page token, received from a previous `ListJobs` call. Provide
      this to retrieve the subsequent page. When paginating, all other
      parameters provided to `ListJobs` must match the call that provided the
      page token.
    parent: Required. The pipeline name. For example:
      `projects/PROJECT_ID/locations/LOCATION_ID/pipelines/PIPELINE_ID`.
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class DatapipelinesProjectsLocationsPipelinesPatchRequest(_messages.Message):
  r"""A DatapipelinesProjectsLocationsPipelinesPatchRequest object.

  Fields:
    googleCloudDatapipelinesV1Pipeline: A GoogleCloudDatapipelinesV1Pipeline
      resource to be passed as the request body.
    name: The pipeline name. For example:
      `projects/PROJECT_ID/locations/LOCATION_ID/pipelines/PIPELINE_ID`. *
      `PROJECT_ID` can contain letters ([A-Za-z]), numbers ([0-9]), hyphens
      (-), colons (:), and periods (.). For more information, see [Identifying
      projects](https://cloud.google.com/resource-manager/docs/creating-
      managing-projects#identifying_projects). * `LOCATION_ID` is the
      canonical ID for the pipeline's location. The list of available
      locations can be obtained by calling
      `google.cloud.location.Locations.ListLocations`. Note that the Data
      Pipelines service is not available in all regions. It depends on Cloud
      Scheduler, an App Engine application, so it's only available in [App
      Engine regions](https://cloud.google.com/about/locations#region). *
      `PIPELINE_ID` is the ID of the pipeline. Must be unique for the selected
      project and location.
    updateMask: The list of fields to be updated.
  """

  googleCloudDatapipelinesV1Pipeline = _messages.MessageField('GoogleCloudDatapipelinesV1Pipeline', 1)
  name = _messages.StringField(2, required=True)
  updateMask = _messages.StringField(3)


class DatapipelinesProjectsLocationsPipelinesRunRequest(_messages.Message):
  r"""A DatapipelinesProjectsLocationsPipelinesRunRequest object.

  Fields:
    googleCloudDatapipelinesV1RunPipelineRequest: A
      GoogleCloudDatapipelinesV1RunPipelineRequest resource to be passed as
      the request body.
    name: Required. The pipeline name. For example:
      `projects/PROJECT_ID/locations/LOCATION_ID/pipelines/PIPELINE_ID`.
  """

  googleCloudDatapipelinesV1RunPipelineRequest = _messages.MessageField('GoogleCloudDatapipelinesV1RunPipelineRequest', 1)
  name = _messages.StringField(2, required=True)


class DatapipelinesProjectsLocationsPipelinesStopRequest(_messages.Message):
  r"""A DatapipelinesProjectsLocationsPipelinesStopRequest object.

  Fields:
    googleCloudDatapipelinesV1StopPipelineRequest: A
      GoogleCloudDatapipelinesV1StopPipelineRequest resource to be passed as
      the request body.
    name: Required. The pipeline name. For example:
      `projects/PROJECT_ID/locations/LOCATION_ID/pipelines/PIPELINE_ID`.
  """

  googleCloudDatapipelinesV1StopPipelineRequest = _messages.MessageField('GoogleCloudDatapipelinesV1StopPipelineRequest', 1)
  name = _messages.StringField(2, required=True)


class GoogleCloudDatapipelinesV1DataflowJobDetails(_messages.Message):
  r"""Pipeline job details specific to the Dataflow API. This is encapsulated
  here to allow for more executors to store their specific details separately.

  Messages:
    ResourceInfoValue: Cached version of all the metrics of interest for the
      job. This value gets stored here when the job is terminated. As long as
      the job is running, this field is populated from the Dataflow API.

  Fields:
    currentWorkers: Output only. The current number of workers used to run the
      jobs. Only set to a value if the job is still running.
    resourceInfo: Cached version of all the metrics of interest for the job.
      This value gets stored here when the job is terminated. As long as the
      job is running, this field is populated from the Dataflow API.
    sdkVersion: Output only. The SDK version used to run the job.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ResourceInfoValue(_messages.Message):
    r"""Cached version of all the metrics of interest for the job. This value
    gets stored here when the job is terminated. As long as the job is
    running, this field is populated from the Dataflow API.

    Messages:
      AdditionalProperty: An additional property for a ResourceInfoValue
        object.

    Fields:
      additionalProperties: Additional properties of type ResourceInfoValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ResourceInfoValue object.

      Fields:
        key: Name of the additional property.
        value: A number attribute.
      """

      key = _messages.StringField(1)
      value = _messages.FloatField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  currentWorkers = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  resourceInfo = _messages.MessageField('ResourceInfoValue', 2)
  sdkVersion = _messages.MessageField('GoogleCloudDatapipelinesV1SdkVersion', 3)


class GoogleCloudDatapipelinesV1FlexTemplateRuntimeEnvironment(_messages.Message):
  r"""The environment values to be set at runtime for a Flex Template.

  Enums:
    FlexrsGoalValueValuesEnum: Set FlexRS goal for the job.
      https://cloud.google.com/dataflow/docs/guides/flexrs
    IpConfigurationValueValuesEnum: Configuration for VM IPs.

  Messages:
    AdditionalUserLabelsValue: Additional user labels to be specified for the
      job. Keys and values must follow the restrictions specified in the
      [labeling restrictions](https://cloud.google.com/compute/docs/labeling-
      resources#restrictions). An object containing a list of key/value pairs.
      Example: `{ "name": "wrench", "mass": "1kg", "count": "3" }`.

  Fields:
    additionalExperiments: Additional experiment flags for the job.
    additionalUserLabels: Additional user labels to be specified for the job.
      Keys and values must follow the restrictions specified in the [labeling
      restrictions](https://cloud.google.com/compute/docs/labeling-
      resources#restrictions). An object containing a list of key/value pairs.
      Example: `{ "name": "wrench", "mass": "1kg", "count": "3" }`.
    enableStreamingEngine: Whether to enable Streaming Engine for the job.
    flexrsGoal: Set FlexRS goal for the job.
      https://cloud.google.com/dataflow/docs/guides/flexrs
    ipConfiguration: Configuration for VM IPs.
    kmsKeyName: Name for the Cloud KMS key for the job. Key format is:
      projects//locations//keyRings//cryptoKeys/
    machineType: The machine type to use for the job. Defaults to the value
      from the template if not specified.
    maxWorkers: The maximum number of Compute Engine instances to be made
      available to your pipeline during execution, from 1 to 1000.
    network: Network to which VMs will be assigned. If empty or unspecified,
      the service will use the network "default".
    numWorkers: The initial number of Compute Engine instances for the job.
    serviceAccountEmail: The email address of the service account to run the
      job as.
    subnetwork: Subnetwork to which VMs will be assigned, if desired. You can
      specify a subnetwork using either a complete URL or an abbreviated path.
      Expected to be of the form "https://www.googleapis.com/compute/v1/projec
      ts/HOST_PROJECT_ID/regions/REGION/subnetworks/SUBNETWORK" or
      "regions/REGION/subnetworks/SUBNETWORK". If the subnetwork is located in
      a Shared VPC network, you must use the complete URL.
    tempLocation: The Cloud Storage path to use for temporary files. Must be a
      valid Cloud Storage URL, beginning with `gs://`.
    workerRegion: The Compute Engine region
      (https://cloud.google.com/compute/docs/regions-zones/regions-zones) in
      which worker processing should occur, e.g. "us-west1". Mutually
      exclusive with worker_zone. If neither worker_region nor worker_zone is
      specified, defaults to the control plane region.
    workerZone: The Compute Engine zone
      (https://cloud.google.com/compute/docs/regions-zones/regions-zones) in
      which worker processing should occur, e.g. "us-west1-a". Mutually
      exclusive with worker_region. If neither worker_region nor worker_zone
      is specified, a zone in the control plane region is chosen based on
      available capacity. If both `worker_zone` and `zone` are set,
      `worker_zone` takes precedence.
    zone: The Compute Engine [availability
      zone](https://cloud.google.com/compute/docs/regions-zones/regions-zones)
      for launching worker instances to run your pipeline. In the future,
      worker_zone will take precedence.
  """

  class FlexrsGoalValueValuesEnum(_messages.Enum):
    r"""Set FlexRS goal for the job.
    https://cloud.google.com/dataflow/docs/guides/flexrs

    Values:
      FLEXRS_UNSPECIFIED: Run in the default mode.
      FLEXRS_SPEED_OPTIMIZED: Optimize for lower execution time.
      FLEXRS_COST_OPTIMIZED: Optimize for lower cost.
    """
    FLEXRS_UNSPECIFIED = 0
    FLEXRS_SPEED_OPTIMIZED = 1
    FLEXRS_COST_OPTIMIZED = 2

  class IpConfigurationValueValuesEnum(_messages.Enum):
    r"""Configuration for VM IPs.

    Values:
      WORKER_IP_UNSPECIFIED: The configuration is unknown, or unspecified.
      WORKER_IP_PUBLIC: Workers should have public IP addresses.
      WORKER_IP_PRIVATE: Workers should have private IP addresses.
    """
    WORKER_IP_UNSPECIFIED = 0
    WORKER_IP_PUBLIC = 1
    WORKER_IP_PRIVATE = 2

  @encoding.MapUnrecognizedFields('additionalProperties')
  class AdditionalUserLabelsValue(_messages.Message):
    r"""Additional user labels to be specified for the job. Keys and values
    must follow the restrictions specified in the [labeling
    restrictions](https://cloud.google.com/compute/docs/labeling-
    resources#restrictions). An object containing a list of key/value pairs.
    Example: `{ "name": "wrench", "mass": "1kg", "count": "3" }`.

    Messages:
      AdditionalProperty: An additional property for a
        AdditionalUserLabelsValue object.

    Fields:
      additionalProperties: Additional properties of type
        AdditionalUserLabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a AdditionalUserLabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  additionalExperiments = _messages.StringField(1, repeated=True)
  additionalUserLabels = _messages.MessageField('AdditionalUserLabelsValue', 2)
  enableStreamingEngine = _messages.BooleanField(3)
  flexrsGoal = _messages.EnumField('FlexrsGoalValueValuesEnum', 4)
  ipConfiguration = _messages.EnumField('IpConfigurationValueValuesEnum', 5)
  kmsKeyName = _messages.StringField(6)
  machineType = _messages.StringField(7)
  maxWorkers = _messages.IntegerField(8, variant=_messages.Variant.INT32)
  network = _messages.StringField(9)
  numWorkers = _messages.IntegerField(10, variant=_messages.Variant.INT32)
  serviceAccountEmail = _messages.StringField(11)
  subnetwork = _messages.StringField(12)
  tempLocation = _messages.StringField(13)
  workerRegion = _messages.StringField(14)
  workerZone = _messages.StringField(15)
  zone = _messages.StringField(16)


class GoogleCloudDatapipelinesV1Job(_messages.Message):
  r"""Definition of the job information maintained by the pipeline. Fields in
  this entity are retrieved from the executor API (e.g. Dataflow API).

  Enums:
    StateValueValuesEnum: The current state of the job.

  Fields:
    createTime: Output only. The time of job creation.
    dataflowJobDetails: All the details that are specific to a Dataflow job.
    endTime: Output only. The time of job termination. This is absent if the
      job is still running.
    id: Output only. The internal ID for the job.
    name: Required. The fully qualified resource name for the job.
    state: The current state of the job.
    status: Status capturing any error code or message related to job creation
      or execution.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""The current state of the job.

    Values:
      STATE_UNSPECIFIED: The job state isn't specified.
      STATE_PENDING: The job is waiting to start execution.
      STATE_RUNNING: The job is executing.
      STATE_DONE: The job has finished execution successfully.
      STATE_FAILED: The job has finished execution with a failure.
      STATE_CANCELLED: The job has been terminated upon user request.
    """
    STATE_UNSPECIFIED = 0
    STATE_PENDING = 1
    STATE_RUNNING = 2
    STATE_DONE = 3
    STATE_FAILED = 4
    STATE_CANCELLED = 5

  createTime = _messages.StringField(1)
  dataflowJobDetails = _messages.MessageField('GoogleCloudDatapipelinesV1DataflowJobDetails', 2)
  endTime = _messages.StringField(3)
  id = _messages.StringField(4)
  name = _messages.StringField(5)
  state = _messages.EnumField('StateValueValuesEnum', 6)
  status = _messages.MessageField('GoogleRpcStatus', 7)


class GoogleCloudDatapipelinesV1LaunchFlexTemplateParameter(_messages.Message):
  r"""Launch Flex Template parameter.

  Messages:
    LaunchOptionsValue: Launch options for this Flex Template job. This is a
      common set of options across languages and templates. This should not be
      used to pass job parameters.
    ParametersValue: The parameters for the Flex Template. Example:
      `{"num_workers":"5"}`
    TransformNameMappingsValue: Use this to pass transform name mappings for
      streaming update jobs. Example:
      `{"oldTransformName":"newTransformName",...}`

  Fields:
    containerSpecGcsPath: Cloud Storage path to a file with a JSON-serialized
      ContainerSpec as content.
    environment: The runtime environment for the Flex Template job.
    jobName: Required. The job name to use for the created job. For an update
      job request, the job name should be the same as the existing running
      job.
    launchOptions: Launch options for this Flex Template job. This is a common
      set of options across languages and templates. This should not be used
      to pass job parameters.
    parameters: The parameters for the Flex Template. Example:
      `{"num_workers":"5"}`
    transformNameMappings: Use this to pass transform name mappings for
      streaming update jobs. Example:
      `{"oldTransformName":"newTransformName",...}`
    update: Set this to true if you are sending a request to update a running
      streaming job. When set, the job name should be the same as the running
      job.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LaunchOptionsValue(_messages.Message):
    r"""Launch options for this Flex Template job. This is a common set of
    options across languages and templates. This should not be used to pass
    job parameters.

    Messages:
      AdditionalProperty: An additional property for a LaunchOptionsValue
        object.

    Fields:
      additionalProperties: Additional properties of type LaunchOptionsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LaunchOptionsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ParametersValue(_messages.Message):
    r"""The parameters for the Flex Template. Example: `{"num_workers":"5"}`

    Messages:
      AdditionalProperty: An additional property for a ParametersValue object.

    Fields:
      additionalProperties: Additional properties of type ParametersValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ParametersValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class TransformNameMappingsValue(_messages.Message):
    r"""Use this to pass transform name mappings for streaming update jobs.
    Example: `{"oldTransformName":"newTransformName",...}`

    Messages:
      AdditionalProperty: An additional property for a
        TransformNameMappingsValue object.

    Fields:
      additionalProperties: Additional properties of type
        TransformNameMappingsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a TransformNameMappingsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  containerSpecGcsPath = _messages.StringField(1)
  environment = _messages.MessageField('GoogleCloudDatapipelinesV1FlexTemplateRuntimeEnvironment', 2)
  jobName = _messages.StringField(3)
  launchOptions = _messages.MessageField('LaunchOptionsValue', 4)
  parameters = _messages.MessageField('ParametersValue', 5)
  transformNameMappings = _messages.MessageField('TransformNameMappingsValue', 6)
  update = _messages.BooleanField(7)


class GoogleCloudDatapipelinesV1LaunchFlexTemplateRequest(_messages.Message):
  r"""A request to launch a Dataflow job from a Flex Template.

  Fields:
    launchParameter: Required. Parameter to launch a job from a Flex Template.
    location: Required. The [regional endpoint]
      (https://cloud.google.com/dataflow/docs/concepts/regional-endpoints) to
      which to direct the request. For example, `us-central1`, `us-west1`.
    projectId: Required. The ID of the Cloud Platform project that the job
      belongs to.
    validateOnly: If true, the request is validated but not actually executed.
      Defaults to false.
  """

  launchParameter = _messages.MessageField('GoogleCloudDatapipelinesV1LaunchFlexTemplateParameter', 1)
  location = _messages.StringField(2)
  projectId = _messages.StringField(3)
  validateOnly = _messages.BooleanField(4)


class GoogleCloudDatapipelinesV1LaunchTemplateParameters(_messages.Message):
  r"""Parameters to provide to the template being launched.

  Messages:
    ParametersValue: The runtime parameters to pass to the job.
    TransformNameMappingValue: Map of transform name prefixes of the job to be
      replaced to the corresponding name prefixes of the new job. Only
      applicable when updating a pipeline.

  Fields:
    environment: The runtime environment for the job.
    jobName: Required. The job name to use for the created job.
    parameters: The runtime parameters to pass to the job.
    transformNameMapping: Map of transform name prefixes of the job to be
      replaced to the corresponding name prefixes of the new job. Only
      applicable when updating a pipeline.
    update: If set, replace the existing pipeline with the name specified by
      jobName with this pipeline, preserving state.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ParametersValue(_messages.Message):
    r"""The runtime parameters to pass to the job.

    Messages:
      AdditionalProperty: An additional property for a ParametersValue object.

    Fields:
      additionalProperties: Additional properties of type ParametersValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ParametersValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class TransformNameMappingValue(_messages.Message):
    r"""Map of transform name prefixes of the job to be replaced to the
    corresponding name prefixes of the new job. Only applicable when updating
    a pipeline.

    Messages:
      AdditionalProperty: An additional property for a
        TransformNameMappingValue object.

    Fields:
      additionalProperties: Additional properties of type
        TransformNameMappingValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a TransformNameMappingValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  environment = _messages.MessageField('GoogleCloudDatapipelinesV1RuntimeEnvironment', 1)
  jobName = _messages.StringField(2)
  parameters = _messages.MessageField('ParametersValue', 3)
  transformNameMapping = _messages.MessageField('TransformNameMappingValue', 4)
  update = _messages.BooleanField(5)


class GoogleCloudDatapipelinesV1LaunchTemplateRequest(_messages.Message):
  r"""A request to launch a template.

  Fields:
    gcsPath: A Cloud Storage path to the template from which to create the
      job. Must be a valid Cloud Storage URL, beginning with 'gs://'.
    launchParameters: The parameters of the template to launch. This should be
      part of the body of the POST request.
    location: The [regional endpoint]
      (https://cloud.google.com/dataflow/docs/concepts/regional-endpoints) to
      which to direct the request.
    projectId: Required. The ID of the Cloud Platform project that the job
      belongs to.
    validateOnly: If true, the request is validated but not actually executed.
      Defaults to false.
  """

  gcsPath = _messages.StringField(1)
  launchParameters = _messages.MessageField('GoogleCloudDatapipelinesV1LaunchTemplateParameters', 2)
  location = _messages.StringField(3)
  projectId = _messages.StringField(4)
  validateOnly = _messages.BooleanField(5)


class GoogleCloudDatapipelinesV1ListJobsResponse(_messages.Message):
  r"""Response message for ListJobs

  Fields:
    jobs: Results that were accessible to the caller. Results are always in
      descending order of job creation date.
    nextPageToken: A token, which can be sent as `page_token` to retrieve the
      next page. If this field is omitted, there are no subsequent pages.
  """

  jobs = _messages.MessageField('GoogleCloudDatapipelinesV1Job', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class GoogleCloudDatapipelinesV1ListPipelinesResponse(_messages.Message):
  r"""Response message for ListPipelines.

  Fields:
    nextPageToken: A token, which can be sent as `page_token` to retrieve the
      next page. If this field is omitted, there are no subsequent pages.
    pipelines: Results that matched the filter criteria and were accessible to
      the caller. Results are always in descending order of pipeline creation
      date.
  """

  nextPageToken = _messages.StringField(1)
  pipelines = _messages.MessageField('GoogleCloudDatapipelinesV1Pipeline', 2, repeated=True)


class GoogleCloudDatapipelinesV1Pipeline(_messages.Message):
  r"""The main pipeline entity and all the necessary metadata for launching
  and managing linked jobs.

  Enums:
    StateValueValuesEnum: Required. The state of the pipeline. When the
      pipeline is created, the state is set to 'PIPELINE_STATE_ACTIVE' by
      default. State changes can be requested by setting the state to
      stopping, paused, or resuming. State cannot be changed through
      UpdatePipeline requests.
    TypeValueValuesEnum: Required. The type of the pipeline. This field
      affects the scheduling of the pipeline and the type of metrics to show
      for the pipeline.

  Messages:
    PipelineSourcesValue: Immutable. The sources of the pipeline (for example,
      Dataplex). The keys and values are set by the corresponding sources
      during pipeline creation.

  Fields:
    createTime: Output only. Immutable. The timestamp when the pipeline was
      initially created. Set by the Data Pipelines service.
    displayName: Required. The display name of the pipeline. It can contain
      only letters ([A-Za-z]), numbers ([0-9]), hyphens (-), and underscores
      (_).
    jobCount: Output only. Number of jobs.
    lastUpdateTime: Output only. Immutable. The timestamp when the pipeline
      was last modified. Set by the Data Pipelines service.
    name: The pipeline name. For example:
      `projects/PROJECT_ID/locations/LOCATION_ID/pipelines/PIPELINE_ID`. *
      `PROJECT_ID` can contain letters ([A-Za-z]), numbers ([0-9]), hyphens
      (-), colons (:), and periods (.). For more information, see [Identifying
      projects](https://cloud.google.com/resource-manager/docs/creating-
      managing-projects#identifying_projects). * `LOCATION_ID` is the
      canonical ID for the pipeline's location. The list of available
      locations can be obtained by calling
      `google.cloud.location.Locations.ListLocations`. Note that the Data
      Pipelines service is not available in all regions. It depends on Cloud
      Scheduler, an App Engine application, so it's only available in [App
      Engine regions](https://cloud.google.com/about/locations#region). *
      `PIPELINE_ID` is the ID of the pipeline. Must be unique for the selected
      project and location.
    pipelineSources: Immutable. The sources of the pipeline (for example,
      Dataplex). The keys and values are set by the corresponding sources
      during pipeline creation.
    scheduleInfo: Internal scheduling information for a pipeline. If this
      information is provided, periodic jobs will be created per the schedule.
      If not, users are responsible for creating jobs externally.
    schedulerServiceAccountEmail: Optional. A service account email to be used
      with the Cloud Scheduler job. If not specified, the default compute
      engine service account will be used.
    state: Required. The state of the pipeline. When the pipeline is created,
      the state is set to 'PIPELINE_STATE_ACTIVE' by default. State changes
      can be requested by setting the state to stopping, paused, or resuming.
      State cannot be changed through UpdatePipeline requests.
    type: Required. The type of the pipeline. This field affects the
      scheduling of the pipeline and the type of metrics to show for the
      pipeline.
    workload: Workload information for creating new jobs.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Required. The state of the pipeline. When the pipeline is created, the
    state is set to 'PIPELINE_STATE_ACTIVE' by default. State changes can be
    requested by setting the state to stopping, paused, or resuming. State
    cannot be changed through UpdatePipeline requests.

    Values:
      STATE_UNSPECIFIED: The pipeline state isn't specified.
      STATE_RESUMING: The pipeline is getting started or resumed. When
        finished, the pipeline state will be 'PIPELINE_STATE_ACTIVE'.
      STATE_ACTIVE: The pipeline is actively running.
      STATE_STOPPING: The pipeline is in the process of stopping. When
        finished, the pipeline state will be 'PIPELINE_STATE_ARCHIVED'.
      STATE_ARCHIVED: The pipeline has been stopped. This is a terminal state
        and cannot be undone.
      STATE_PAUSED: The pipeline is paused. This is a non-terminal state. When
        the pipeline is paused, it will hold processing jobs, but can be
        resumed later. For a batch pipeline, this means pausing the scheduler
        job. For a streaming pipeline, creating a job snapshot to resume from
        will give the same effect.
    """
    STATE_UNSPECIFIED = 0
    STATE_RESUMING = 1
    STATE_ACTIVE = 2
    STATE_STOPPING = 3
    STATE_ARCHIVED = 4
    STATE_PAUSED = 5

  class TypeValueValuesEnum(_messages.Enum):
    r"""Required. The type of the pipeline. This field affects the scheduling
    of the pipeline and the type of metrics to show for the pipeline.

    Values:
      PIPELINE_TYPE_UNSPECIFIED: The pipeline type isn't specified.
      PIPELINE_TYPE_BATCH: A batch pipeline. It runs jobs on a specific
        schedule, and each job will automatically terminate once execution is
        finished.
      PIPELINE_TYPE_STREAMING: A streaming pipeline. The underlying job is
        continuously running until it is manually terminated by the user. This
        type of pipeline doesn't have a schedule to run on, and the linked job
        gets created when the pipeline is created.
    """
    PIPELINE_TYPE_UNSPECIFIED = 0
    PIPELINE_TYPE_BATCH = 1
    PIPELINE_TYPE_STREAMING = 2

  @encoding.MapUnrecognizedFields('additionalProperties')
  class PipelineSourcesValue(_messages.Message):
    r"""Immutable. The sources of the pipeline (for example, Dataplex). The
    keys and values are set by the corresponding sources during pipeline
    creation.

    Messages:
      AdditionalProperty: An additional property for a PipelineSourcesValue
        object.

    Fields:
      additionalProperties: Additional properties of type PipelineSourcesValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a PipelineSourcesValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  createTime = _messages.StringField(1)
  displayName = _messages.StringField(2)
  jobCount = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  lastUpdateTime = _messages.StringField(4)
  name = _messages.StringField(5)
  pipelineSources = _messages.MessageField('PipelineSourcesValue', 6)
  scheduleInfo = _messages.MessageField('GoogleCloudDatapipelinesV1ScheduleSpec', 7)
  schedulerServiceAccountEmail = _messages.StringField(8)
  state = _messages.EnumField('StateValueValuesEnum', 9)
  type = _messages.EnumField('TypeValueValuesEnum', 10)
  workload = _messages.MessageField('GoogleCloudDatapipelinesV1Workload', 11)


class GoogleCloudDatapipelinesV1RunPipelineRequest(_messages.Message):
  r"""Request message for RunPipeline"""


class GoogleCloudDatapipelinesV1RunPipelineResponse(_messages.Message):
  r"""Response message for RunPipeline

  Fields:
    job: Job that was created as part of RunPipeline operation.
  """

  job = _messages.MessageField('GoogleCloudDatapipelinesV1Job', 1)


class GoogleCloudDatapipelinesV1RuntimeEnvironment(_messages.Message):
  r"""The environment values to set at runtime.

  Enums:
    IpConfigurationValueValuesEnum: Configuration for VM IPs.

  Messages:
    AdditionalUserLabelsValue: Additional user labels to be specified for the
      job. Keys and values should follow the restrictions specified in the
      [labeling restrictions](https://cloud.google.com/compute/docs/labeling-
      resources#restrictions) page. An object containing a list of key/value
      pairs. Example: { "name": "wrench", "mass": "1kg", "count": "3" }.

  Fields:
    additionalExperiments: Additional experiment flags for the job.
    additionalUserLabels: Additional user labels to be specified for the job.
      Keys and values should follow the restrictions specified in the
      [labeling restrictions](https://cloud.google.com/compute/docs/labeling-
      resources#restrictions) page. An object containing a list of key/value
      pairs. Example: { "name": "wrench", "mass": "1kg", "count": "3" }.
    bypassTempDirValidation: Whether to bypass the safety checks for the job's
      temporary directory. Use with caution.
    enableStreamingEngine: Whether to enable Streaming Engine for the job.
    ipConfiguration: Configuration for VM IPs.
    kmsKeyName: Name for the Cloud KMS key for the job. The key format is:
      projects//locations//keyRings//cryptoKeys/
    machineType: The machine type to use for the job. Defaults to the value
      from the template if not specified.
    maxWorkers: The maximum number of Compute Engine instances to be made
      available to your pipeline during execution, from 1 to 1000.
    network: Network to which VMs will be assigned. If empty or unspecified,
      the service will use the network "default".
    numWorkers: The initial number of Compute Engine instances for the job.
    serviceAccountEmail: The email address of the service account to run the
      job as.
    subnetwork: Subnetwork to which VMs will be assigned, if desired. You can
      specify a subnetwork using either a complete URL or an abbreviated path.
      Expected to be of the form "https://www.googleapis.com/compute/v1/projec
      ts/HOST_PROJECT_ID/regions/REGION/subnetworks/SUBNETWORK" or
      "regions/REGION/subnetworks/SUBNETWORK". If the subnetwork is located in
      a Shared VPC network, you must use the complete URL.
    tempLocation: The Cloud Storage path to use for temporary files. Must be a
      valid Cloud Storage URL, beginning with `gs://`.
    workerRegion: The Compute Engine region
      (https://cloud.google.com/compute/docs/regions-zones/regions-zones) in
      which worker processing should occur, e.g. "us-west1". Mutually
      exclusive with worker_zone. If neither worker_region nor worker_zone is
      specified, default to the control plane's region.
    workerZone: The Compute Engine zone
      (https://cloud.google.com/compute/docs/regions-zones/regions-zones) in
      which worker processing should occur, e.g. "us-west1-a". Mutually
      exclusive with worker_region. If neither worker_region nor worker_zone
      is specified, a zone in the control plane's region is chosen based on
      available capacity. If both `worker_zone` and `zone` are set,
      `worker_zone` takes precedence.
    zone: The Compute Engine [availability
      zone](https://cloud.google.com/compute/docs/regions-zones/regions-zones)
      for launching worker instances to run your pipeline. In the future,
      worker_zone will take precedence.
  """

  class IpConfigurationValueValuesEnum(_messages.Enum):
    r"""Configuration for VM IPs.

    Values:
      WORKER_IP_UNSPECIFIED: The configuration is unknown, or unspecified.
      WORKER_IP_PUBLIC: Workers should have public IP addresses.
      WORKER_IP_PRIVATE: Workers should have private IP addresses.
    """
    WORKER_IP_UNSPECIFIED = 0
    WORKER_IP_PUBLIC = 1
    WORKER_IP_PRIVATE = 2

  @encoding.MapUnrecognizedFields('additionalProperties')
  class AdditionalUserLabelsValue(_messages.Message):
    r"""Additional user labels to be specified for the job. Keys and values
    should follow the restrictions specified in the [labeling
    restrictions](https://cloud.google.com/compute/docs/labeling-
    resources#restrictions) page. An object containing a list of key/value
    pairs. Example: { "name": "wrench", "mass": "1kg", "count": "3" }.

    Messages:
      AdditionalProperty: An additional property for a
        AdditionalUserLabelsValue object.

    Fields:
      additionalProperties: Additional properties of type
        AdditionalUserLabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a AdditionalUserLabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  additionalExperiments = _messages.StringField(1, repeated=True)
  additionalUserLabels = _messages.MessageField('AdditionalUserLabelsValue', 2)
  bypassTempDirValidation = _messages.BooleanField(3)
  enableStreamingEngine = _messages.BooleanField(4)
  ipConfiguration = _messages.EnumField('IpConfigurationValueValuesEnum', 5)
  kmsKeyName = _messages.StringField(6)
  machineType = _messages.StringField(7)
  maxWorkers = _messages.IntegerField(8, variant=_messages.Variant.INT32)
  network = _messages.StringField(9)
  numWorkers = _messages.IntegerField(10, variant=_messages.Variant.INT32)
  serviceAccountEmail = _messages.StringField(11)
  subnetwork = _messages.StringField(12)
  tempLocation = _messages.StringField(13)
  workerRegion = _messages.StringField(14)
  workerZone = _messages.StringField(15)
  zone = _messages.StringField(16)


class GoogleCloudDatapipelinesV1ScheduleSpec(_messages.Message):
  r"""Details of the schedule the pipeline runs on.

  Fields:
    nextJobTime: Output only. When the next Scheduler job is going to run.
    schedule: Unix-cron format of the schedule. This information is retrieved
      from the linked Cloud Scheduler.
    timeZone: Timezone ID. This matches the timezone IDs used by the Cloud
      Scheduler API. If empty, UTC time is assumed.
  """

  nextJobTime = _messages.StringField(1)
  schedule = _messages.StringField(2)
  timeZone = _messages.StringField(3)


class GoogleCloudDatapipelinesV1SdkVersion(_messages.Message):
  r"""The version of the SDK used to run the job.

  Enums:
    SdkSupportStatusValueValuesEnum: The support status for this SDK version.

  Fields:
    sdkSupportStatus: The support status for this SDK version.
    version: The version of the SDK used to run the job.
    versionDisplayName: A readable string describing the version of the SDK.
  """

  class SdkSupportStatusValueValuesEnum(_messages.Enum):
    r"""The support status for this SDK version.

    Values:
      UNKNOWN: Dataflow is unaware of this version.
      SUPPORTED: This is a known version of an SDK, and is supported.
      STALE: A newer version of the SDK exists, and an update is recommended.
      DEPRECATED: This version of the SDK is deprecated and will eventually be
        unsupported.
      UNSUPPORTED: Support for this SDK version has ended and it should no
        longer be used.
    """
    UNKNOWN = 0
    SUPPORTED = 1
    STALE = 2
    DEPRECATED = 3
    UNSUPPORTED = 4

  sdkSupportStatus = _messages.EnumField('SdkSupportStatusValueValuesEnum', 1)
  version = _messages.StringField(2)
  versionDisplayName = _messages.StringField(3)


class GoogleCloudDatapipelinesV1StopPipelineRequest(_messages.Message):
  r"""Request message for StopPipeline."""


class GoogleCloudDatapipelinesV1Workload(_messages.Message):
  r"""Workload details for creating the pipeline jobs.

  Fields:
    dataflowFlexTemplateRequest: Template information and additional
      parameters needed to launch a Dataflow job using the flex launch API.
    dataflowLaunchTemplateRequest: Template information and additional
      parameters needed to launch a Dataflow job using the standard launch
      API.
  """

  dataflowFlexTemplateRequest = _messages.MessageField('GoogleCloudDatapipelinesV1LaunchFlexTemplateRequest', 1)
  dataflowLaunchTemplateRequest = _messages.MessageField('GoogleCloudDatapipelinesV1LaunchTemplateRequest', 2)


class GoogleProtobufEmpty(_messages.Message):
  r"""A generic empty message that you can re-use to avoid defining duplicated
  empty messages in your APIs. A typical example is to use it as the request
  or the response type of an API method. For instance: service Foo { rpc
  Bar(google.protobuf.Empty) returns (google.protobuf.Empty); }
  """



class GoogleRpcStatus(_messages.Message):
  r"""The `Status` type defines a logical error model that is suitable for
  different programming environments, including REST APIs and RPC APIs. It is
  used by [gRPC](https://github.com/grpc). Each `Status` message contains
  three pieces of data: error code, error message, and error details. You can
  find out more about this error model and how to work with it in the [API
  Design Guide](https://cloud.google.com/apis/design/errors).

  Messages:
    DetailsValueListEntry: A DetailsValueListEntry object.

  Fields:
    code: The status code, which should be an enum value of google.rpc.Code.
    details: A list of messages that carry the error details. There is a
      common set of message types for APIs to use.
    message: A developer-facing error message, which should be in English. Any
      user-facing error message should be localized and sent in the
      google.rpc.Status.details field, or localized by the client.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DetailsValueListEntry(_messages.Message):
    r"""A DetailsValueListEntry object.

    Messages:
      AdditionalProperty: An additional property for a DetailsValueListEntry
        object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DetailsValueListEntry object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  code = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  details = _messages.MessageField('DetailsValueListEntry', 2, repeated=True)
  message = _messages.StringField(3)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command utilities for clusters commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from apitools.base.py import list_pager
from googlecloudsdk.api_lib.container.gkemulticloud import util as api_util
from googlecloudsdk.calliope import base
from googlecloudsdk.core import exceptions
from googlecloudsdk.core import properties

CLUSTERS_FORMAT = """\
  table(
    name.basename(),
    awsRegion,
    controlPlane.version:label=CONTROL_PLANE_VERSION,
    controlPlane.instanceType,
    state)"""


class UnsupportedPropertyError(exceptions.Error):
  """Class for errors by unsupported properties."""


class Client(object):
  """Client for managing GKE clusters on AWS."""

  def __init__(self, client=None, messages=None, track=base.ReleaseTrack.GA):
    self.client = client or api_util.GetClientInstance(release_track=track)
    self.messages = messages or api_util.GetMessagesModule(release_track=track)
    self.service = self.client.projects_locations_awsClusters
    self.track = track
    self.version = api_util.GetApiVersionForTrack(track).capitalize()

  def _AddAwsCluster(self, req):
    msg = 'GoogleCloudGkemulticloud{}AwsCluster'.format(self.version)
    attr = 'googleCloudGkemulticloud{}AwsCluster'.format(self.version)
    cluster = getattr(self.messages, msg)()
    setattr(req, attr, cluster)
    return cluster

  def _AddAwsNetworking(self, req):
    msg = 'GoogleCloudGkemulticloud{}AwsClusterNetworking'.format(self.version)
    net = getattr(self.messages, msg)()
    req.networking = net
    return net

  def _AddAwsControlPlane(self, req):
    msg = 'GoogleCloudGkemulticloud{}AwsControlPlane'.format(self.version)
    cp = getattr(self.messages, msg)()
    req.controlPlane = cp
    return cp

  def _AddAwsAuthorization(self, req):
    msg = 'GoogleCloudGkemulticloud{}AwsAuthorization'.format(self.version)
    a = getattr(self.messages, msg)()
    req.authorization = a
    return a

  def _AddAwsServicesAuthentication(self, req):
    msg = 'GoogleCloudGkemulticloud{}AwsServicesAuthentication'.format(
        self.version)
    sa = getattr(self.messages, msg)()
    req.awsServicesAuthentication = sa
    return sa

  def _AddAwsProxyConfig(self, req):
    msg = 'GoogleCloudGkemulticloud{}AwsProxyConfig'.format(self.version)
    pc = getattr(self.messages, msg)()
    req.proxyConfig = pc
    return pc

  def _CreateAwsClusterUser(self, username):
    msg = 'GoogleCloudGkemulticloud{}AwsClusterUser'.format(self.version)
    return getattr(self.messages, msg)(username=username)

  def _CreateAwsVolumeTemplate(self, size_gib, volume_type, iops, kms_key_arn):
    msg = 'GoogleCloudGkemulticloud{}AwsVolumeTemplate'.format(self.version)
    return getattr(self.messages, msg)(
        sizeGib=size_gib,
        volumeType=volume_type,
        iops=iops,
        kmsKeyArn=kms_key_arn)

  def _CreateAwsDatabaseEncryption(self, database_encryption_key):
    msg = 'GoogleCloudGkemulticloud{}AwsDatabaseEncryption'.format(self.version)
    return getattr(self.messages, msg)(kmsKeyArn=database_encryption_key)

  def _CreateAwsConfigEncryption(self, config_encryption_key):
    msg = 'GoogleCloudGkemulticloud{}AwsConfigEncryption'.format(self.version)
    return getattr(self.messages, msg)(kmsKeyArn=config_encryption_key)

  def _CreateAwsServicesAuthentication(self, role_arn, role_session_name):
    msg = 'GoogleCloudGkemulticloud{}AwsServicesAuthentication'.format(
        self.version)
    return getattr(self.messages, msg)(
        roleArn=role_arn, roleSessionName=role_session_name)

  def _CreateAwsSshConfig(self, key_pair_name):
    msg = 'GoogleCloudGkemulticloud{}AwsSshConfig'.format(self.version)
    return getattr(self.messages, msg)(ec2KeyPair=key_pair_name)

  def _CreateAwsProxyConfig(self, secret_arn, secret_version_id):
    msg = 'GoogleCloudGkemulticloud{}AwsProxyConfig'.format(self.version)
    return getattr(self.messages, msg)(
        secretArn=secret_arn, secretVersion=secret_version_id)

  def _CreateFleet(self, fleet_project):
    msg = 'GoogleCloudGkemulticloud{}Fleet'.format(self.version)
    return getattr(self.messages, msg)(project=fleet_project)

  def _CreateAwsInstancePlacement(self, instance_placement_key):
    msg = 'GoogleCloudGkemulticloud{}AwsInstancePlacement'.format(self.version)
    return getattr(self.messages, msg)(tenancy=instance_placement_key)

  def Create(self, cluster_ref, args):
    """Creates an Anthos cluster on AWS."""
    validate_only = getattr(args, 'validate_only', False)
    req = self.messages.GkemulticloudProjectsLocationsAwsClustersCreateRequest(
        awsClusterId=cluster_ref.awsClustersId,
        parent=cluster_ref.Parent().RelativeName(),
        validateOnly=validate_only)

    c = self._AddAwsCluster(req)
    c.name = cluster_ref.awsClustersId
    c.awsRegion = args.aws_region
    if args.fleet_project:
      c.fleet = self._CreateFleet(args.fleet_project)
    if args.logging:
      c.loggingConfig = args.logging

    cp = self._AddAwsControlPlane(c)
    cp.subnetIds.extend(args.subnet_ids)
    cp.iamInstanceProfile = args.iam_instance_profile
    cp.version = args.cluster_version
    cp.instanceType = args.instance_type
    cp.mainVolume = self._CreateAwsVolumeTemplate(args.main_volume_size,
                                                  args.main_volume_type,
                                                  args.main_volume_iops,
                                                  args.main_volume_kms_key_arn)
    cp.rootVolume = self._CreateAwsVolumeTemplate(args.root_volume_size,
                                                  args.root_volume_type,
                                                  args.root_volume_iops,
                                                  args.root_volume_kms_key_arn)
    cp.databaseEncryption = self._CreateAwsDatabaseEncryption(
        args.database_encryption_kms_key_arn)
    cp.awsServicesAuthentication = self._CreateAwsServicesAuthentication(
        args.role_arn, args.role_session_name)
    cp.sshConfig = self._CreateAwsSshConfig(args.ssh_ec2_key_pair)
    if args.security_group_ids:
      cp.securityGroupIds.extend(args.security_group_ids)
    if args.proxy_secret_arn and args.proxy_secret_version_id:
      cp.proxyConfig = self._CreateAwsProxyConfig(args.proxy_secret_arn,
                                                  args.proxy_secret_version_id)
    if args.config_encryption_kms_key_arn:
      cp.configEncryption = self._CreateAwsConfigEncryption(
          args.config_encryption_kms_key_arn)
    if args.instance_placement:
      cp.instancePlacement = self._CreateAwsInstancePlacement(
          args.instance_placement)

    net = self._AddAwsNetworking(c)
    net.vpcId = args.vpc_id
    net.podAddressCidrBlocks.append(args.pod_address_cidr_blocks)
    net.serviceAddressCidrBlocks.append(args.service_address_cidr_blocks)

    if args.tags:
      tag_type = type(cp).TagsValue.AdditionalProperty
      cp.tags = type(cp).TagsValue(additionalProperties=[
          tag_type(key=k, value=v) for k, v in args.tags.items()
      ])

    a = self._AddAwsAuthorization(c)
    # TODO(b/197350917): Remove the restriction
    # once the feature is implemented.
    # This is because gcloud may be authorized by an account different
    # from core/account. Check if auth/credential_file_override is set.
    if properties.VALUES.auth.credential_file_override.IsExplicitlySet():
      raise UnsupportedPropertyError(
          'The property [auth/credential_file_override] '
          'is not supported by this command.')
    if args.admin_users:
      for username in args.admin_users:
        a.adminUsers.append(self._CreateAwsClusterUser(username))
    else:
      username = properties.VALUES.core.account.GetOrFail()
      a.adminUsers.append(self._CreateAwsClusterUser(username))

    return self.service.Create(req)

  def Delete(self, cluster_ref, args):
    """Delete an AWS cluster."""
    validate_only = getattr(args, 'validate_only', False)
    req = self.messages.GkemulticloudProjectsLocationsAwsClustersDeleteRequest(
        name=cluster_ref.RelativeName(), validateOnly=validate_only)

    return self.service.Delete(req)

  def Get(self, cluster_ref):
    """Get an AWS cluster."""
    req = self.messages.GkemulticloudProjectsLocationsAwsClustersGetRequest(
        name=cluster_ref.RelativeName())
    return self.service.Get(req)

  def GenerateAccessToken(self, cluster_ref):
    """Get an access token for an AWS cluster."""
    req = self.messages.GkemulticloudProjectsLocationsAwsClustersGenerateAwsAccessTokenRequest(
        awsCluster=cluster_ref.RelativeName())
    return self.service.GenerateAwsAccessToken(req)

  def List(self, region_ref):
    """List AWS clusters."""
    req = self.messages.GkemulticloudProjectsLocationsAwsClustersListRequest(
        parent=region_ref.RelativeName())
    for cluster in list_pager.YieldFromList(
        service=self.service,
        request=req,
        field='awsClusters',
        batch_size_attribute='pageSize'):
      yield cluster

  def Update(self, cluster_ref, args):
    """Updates an Anthos cluster on AWS.

    Args:
      cluster_ref: gkemulticloud.GoogleCloudGkemulticloudV1AwsCluster object.
      args: argparse.Namespace, Arguments parsed from the command.

    Returns:
      Response to the update request.
    """
    validate_only = getattr(args, 'validate_only', False)
    req = self.messages.GkemulticloudProjectsLocationsAwsClustersPatchRequest(
        name=cluster_ref.RelativeName(), validateOnly=validate_only)

    update_mask = []
    c = self._AddAwsCluster(req)
    cp = self._AddAwsControlPlane(c)
    a = self._AddAwsAuthorization(c)
    if args.cluster_version:
      cp.version = args.cluster_version
      update_mask.append('control_plane.version')
    if args.instance_type:
      cp.instanceType = args.instance_type
      update_mask.append('control_plane.instance_type')
    if args.config_encryption_kms_key_arn:
      cp.configEncryption = self._CreateAwsConfigEncryption(
          args.config_encryption_kms_key_arn)
      update_mask.append('control_plane.config_encryption.kms_key_arn')
    if args.clear_security_group_ids is not None:
      update_mask.append('control_plane.security_group_ids')
    elif args.security_group_ids:
      cp.securityGroupIds.extend(args.security_group_ids)
      update_mask.append('control_plane.security_group_ids')

    cp.rootVolume = self._CreateAwsVolumeTemplate(args.root_volume_size,
                                                  args.root_volume_type,
                                                  args.root_volume_iops,
                                                  args.root_volume_kms_key_arn)
    if args.root_volume_size:
      update_mask.append('control_plane.root_volume.size_gib')
    if args.root_volume_type:
      update_mask.append('control_plane.root_volume.volume_type')
    if args.root_volume_iops is not None:
      update_mask.append('control_plane.root_volume.iops')
    if args.root_volume_kms_key_arn is not None:
      update_mask.append('control_plane.root_volume.kms_key_arn')

    services_auth = self._AddAwsServicesAuthentication(cp)
    if args.role_arn:
      services_auth.roleArn = args.role_arn
      update_mask.append('control_plane.aws_services_authentication.role_arn')
    if args.role_session_name:
      services_auth.roleSessionName = args.role_session_name
      update_mask.append(
          'control_plane.aws_services_authentication.role_session_name')
    if args.admin_users:
      for username in args.admin_users:
        a.adminUsers.append(self._CreateAwsClusterUser(username))
      update_mask.append('authorization.admin_users')

    if args.clear_proxy_config is not None:
      update_mask.append('control_plane.proxy_config')
    else:
      proxy_config = self._AddAwsProxyConfig(cp)
      if args.proxy_secret_arn:
        proxy_config.secretArn = args.proxy_secret_arn
        update_mask.append('control_plane.proxy_config.secret_arn')
      if args.proxy_secret_version_id:
        proxy_config.secretVersion = args.proxy_secret_version_id
        update_mask.append('control_plane.proxy_config.secret_version')

    req.updateMask = ','.join(update_mask)

    return self.service.Patch(req)



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""services list command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.services import serviceusage
from googlecloudsdk.calliope import base


class List(base.ListCommand):
  """List service quota metrics for a consumer.

  This command lists the service quota metrics for a consumer. The supported
  consumers can be projects, folders, or organizations.

  ## EXAMPLES

  To list the quota metrics for service 'example.googleapis.com' and consumer
  'projects/12321', run:

    $ {command} --service=example.googleapis.com --consumer=projects/12321

  To list the quota metrics for service 'example.googleapis.com' and consumer
  'projects/hello-world', run:

    $ {command} --service=example.googleapis.com --consumer=projects/helloworld

  To list the quota metrics for service 'example.googleapis.com' and consumer
  'folders/12345', run:

    $ {command} --service=example.googleapis.com --consumer=folders/12345

  To list the quota metrics for service 'example.googleapis.com' and consumer
  'organizations/54321', run:

    $ {command} --service=example.googleapis.com --consumer=organizations/54321
  """

  @staticmethod
  def Args(parser):
    """Args is called by calliope to gather arguments for this command.

    Args:
      parser: An argparse parser that you can use to add arguments that go on
        the command line after this command. Positional arguments are allowed.
    """
    parser.add_argument(
        '--service',
        required=True,
        metavar='SERVICE',
        help='The service to list metrics for.')

    parser.add_argument(
        '--consumer',
        required=True,
        metavar='CONSUMER',
        help='The consumer to list metrics for.')

  def Run(self, args):
    """Run command.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
        with.

    Returns:
      The list of quota metrics for the service and consumer.
    """
    metrics = serviceusage.ListQuotaMetrics(args.consumer, args.service,
                                            args.page_size, args.limit)
    return [self.delete_resource_name(m) for m in metrics]

  @staticmethod
  def delete_resource_name(metric):
    """Delete the name fields from metric message.

    Args:
      metric: The quota metric message.

    Returns:
      The updated metric message.
    """
    metric.reset('name')
    for l in metric.consumerQuotaLimits:
      l.reset('name')
    return metric



# Copyright 2018 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This package calculates the digest of an image.

The format this tool *expects* to deal with is proprietary.
Image digests aren't stable upon gzip implementation/configuration.
This tool is expected to be only self-consistent.
"""

from __future__ import absolute_import
from __future__ import print_function

import argparse
import logging
import sys

from containerregistry.client.v2_2 import docker_image as v2_2_image
from containerregistry.client.v2_2 import oci_compat
from containerregistry.tools import logging_setup

from six.moves import zip  # pylint: disable=redefined-builtin

parser = argparse.ArgumentParser(
    description='Calculate digest for a container image.')

parser.add_argument(
    '--tarball', action='store', help='An optional legacy base image tarball.')

parser.add_argument(
    '--output-digest',
    required=True,
    action='store',
    help='Filename to store digest in.')

parser.add_argument(
    '--config',
    action='store',
    help='The path to the file storing the image config.')

parser.add_argument(
    '--manifest',
    action='store',
    help='The path to the file storing the image manifest.')

parser.add_argument(
    '--digest',
    action='append',
    help='The list of layer digest filenames in order.')

parser.add_argument(
    '--layer', action='append', help='The list of layer filenames in order.')

parser.add_argument(
    '--oci', action='store_true', help='Image has an OCI Manifest.')


def main():
  logging_setup.DefineCommandLineArgs(parser)
  args = parser.parse_args()
  logging_setup.Init(args=args)

  if not args.config and (args.layer or args.digest):
    logging.fatal(
        'Using --layer or --digest requires --config to be specified.')
    sys.exit(1)

  if not args.config and not args.tarball:
    logging.fatal('Either --config or --tarball must be specified.')
    sys.exit(1)

  # If config is specified, use that.  Otherwise, fallback on reading
  # the config from the tarball.
  config = args.config
  manifest = args.manifest
  if args.config:
    logging.info('Reading config from %r', args.config)
    with open(args.config, 'r') as reader:
      config = reader.read()
  elif args.tarball:
    logging.info('Reading config from tarball %r', args.tarball)
    with v2_2_image.FromTarball(args.tarball) as base:
      config = base.config_file()

  if args.manifest:
    with open(args.manifest, 'r') as reader:
      manifest = reader.read()

  if len(args.digest or []) != len(args.layer or []):
    logging.fatal('--digest and --layer must have matching lengths.')
    sys.exit(1)

  logging.info('Loading v2.2 image from disk ...')
  with v2_2_image.FromDisk(
      config,
      list(zip(args.digest or [], args.layer or [])),
      legacy_base=args.tarball,
      foreign_layers_manifest=manifest) as v2_2_img:

    try:
      if args.oci:
        with oci_compat.OCIFromV22(v2_2_img) as oci_img:
          digest = oci_img.digest()
      else:
        digest = v2_2_img.digest()

      with open(args.output_digest, 'w+') as digest_file:
        digest_file.write(digest)
    # pylint: disable=broad-except
    except Exception as e:
      logging.fatal('Error getting digest: %s', e)
      sys.exit(1)


if __name__ == '__main__':
  main()



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Optional, Mapping

from google.api_core.client_options import ClientOptions
from google.auth.credentials import Credentials
from google.cloud.pubsub_v1.types import BatchSettings

from google.cloud.pubsublite.cloudpubsub.internal.async_publisher_impl import (
    AsyncSinglePublisherImpl,
)
from google.cloud.pubsublite.cloudpubsub.internal.publisher_impl import (
    SinglePublisherImpl,
)
from google.cloud.pubsublite.cloudpubsub.internal.single_publisher import (
    AsyncSinglePublisher,
    SinglePublisher,
)
from google.cloud.pubsublite.internal.wire.make_publisher import (
    make_publisher as make_wire_publisher,
    DEFAULT_BATCHING_SETTINGS as WIRE_DEFAULT_BATCHING,
)
from google.cloud.pubsublite.internal.wire.merge_metadata import merge_metadata
from google.cloud.pubsublite.internal.wire.pubsub_context import pubsub_context
from google.cloud.pubsublite.types import TopicPath


DEFAULT_BATCHING_SETTINGS = WIRE_DEFAULT_BATCHING


def make_async_publisher(
    topic: TopicPath,
    transport: str,
    per_partition_batching_settings: Optional[BatchSettings] = None,
    credentials: Optional[Credentials] = None,
    client_options: Optional[ClientOptions] = None,
    metadata: Optional[Mapping[str, str]] = None,
) -> AsyncSinglePublisher:
    """
    Make a new publisher for the given topic.

    Args:
      topic: The topic to publish to.
      transport: The transport type to use.
      per_partition_batching_settings: Settings for batching messages on each partition. The default is reasonable for most cases.
      credentials: The credentials to use to connect. GOOGLE_DEFAULT_CREDENTIALS is used if None.
      client_options: Other options to pass to the client. Note that if you pass any you must set api_endpoint.
      metadata: Additional metadata to send with the RPC.

    Returns:
      A new AsyncPublisher.

    Throws:
      GoogleApiCallException on any error determining topic structure.
    """
    metadata = merge_metadata(pubsub_context(framework="CLOUD_PUBSUB_SHIM"), metadata)

    def underlying_factory():
        return make_wire_publisher(
            topic=topic,
            transport=transport,
            per_partition_batching_settings=per_partition_batching_settings,
            credentials=credentials,
            client_options=client_options,
            metadata=metadata,
        )

    return AsyncSinglePublisherImpl(underlying_factory)


def make_publisher(
    topic: TopicPath,
    transport: str,
    per_partition_batching_settings: Optional[BatchSettings] = None,
    credentials: Optional[Credentials] = None,
    client_options: Optional[ClientOptions] = None,
    metadata: Optional[Mapping[str, str]] = None,
) -> SinglePublisher:
    """
    Make a new publisher for the given topic.

    Args:
      topic: The topic to publish to.
      transport: The transport type to use.
      per_partition_batching_settings: Settings for batching messages on each partition. The default is reasonable for most cases.
      credentials: The credentials to use to connect. GOOGLE_DEFAULT_CREDENTIALS is used if None.
      client_options: Other options to pass to the client. Note that if you pass any you must set api_endpoint.
      metadata: Additional metadata to send with the RPC.

    Returns:
      A new Publisher.

    Throws:
      GoogleApiCallException on any error determining topic structure.
    """
    return SinglePublisherImpl(
        make_async_publisher(
            topic=topic,
            transport=transport,
            per_partition_batching_settings=per_partition_batching_settings,
            credentials=credentials,
            client_options=client_options,
            metadata=metadata,
        )
    )



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Set-policy command for the Resource Settings CLI."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from apitools.base.py import exceptions as api_exceptions
from argcomplete import completers
from googlecloudsdk.api_lib.resourcesettings import service
from googlecloudsdk.api_lib.resourcesettings import utils as api_utils
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.resource_settings import exceptions
from googlecloudsdk.command_lib.resource_settings import utils


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.GA)
class SetValue(base.Command):
  r"""Update the value of a resource setting.

  Update the value of a resource setting.

  ## EXAMPLES

  To set the resource setting from the file with the path ``./sample_path'',
  run:

    $ {command} --value-file="./test_input.json"
  """

  @staticmethod
  def Args(parser):
    parser.add_argument(
        '--value-file',
        metavar='value-file',
        completer=completers.FilesCompleter,
        required=True,
        help='Path to JSON or YAML file that contains the resource setting.')

  def Run(self, args):
    """Creates or updates a setting from a JSON or YAML file.

    Args:
      args: argparse.Namespace, An object that contains the values for the
        arguments specified in the Args method.

    Returns:
      The created or updated setting.
    """
    settings_message = service.ResourceSettingsMessages()

    input_setting = utils.GetMessageFromFile(
        args.value_file, settings_message.GoogleCloudResourcesettingsV1Setting)

    if not input_setting.name:
      raise exceptions.InvalidInputError(
          'Name field not present in the resource setting.')

    if not utils.ValidateSettingPath(input_setting.name):
      raise exceptions.InvalidInputError('Name field has invalid syntax')

    resource_type = utils.GetResourceTypeFromString(input_setting.name)
    settings_service = api_utils.GetServiceFromResourceType(resource_type)

    # Syntax: [organizations|folders|projects]/{resource_id}/
    #          settings/{setting_name}
    setting_path = input_setting.name

    etag = input.etag if hasattr(input, 'etag') else None
    update_request = api_utils.GetPatchRequestFromResourceType(
        resource_type, setting_path, input_setting.localValue, etag)
    update_response = settings_service.Patch(update_request)
    return update_response



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Resource definitions for cloud platform apis."""

import enum


BASE_URL = 'https://dialogflow.googleapis.com/v2/'
DOCS_URL = 'https://cloud.google.com/dialogflow/'


class Collections(enum.Enum):
  """Collections for all supported apis."""

  PROJECTS = (
      'projects',
      'projects/{projectsId}',
      {},
      ['projectsId'],
      True
  )
  PROJECTS_AGENT = (
      'projects.agent',
      'projects/{projectsId}/agent',
      {},
      ['projectsId'],
      True
  )
  PROJECTS_AGENT_ENTITYTYPES = (
      'projects.agent.entityTypes',
      '{+name}',
      {
          '':
              'projects/{projectsId}/agent/entityTypes/{entityTypesId}',
      },
      ['name'],
      True
  )
  PROJECTS_AGENT_ENVIRONMENTS = (
      'projects.agent.environments',
      '{+name}',
      {
          '':
              'projects/{projectsId}/agent/environments/{environmentsId}',
      },
      ['name'],
      True
  )
  PROJECTS_AGENT_ENVIRONMENTS_USERS = (
      'projects.agent.environments.users',
      'projects/{projectsId}/agent/environments/{environmentsId}/users/'
      '{usersId}',
      {},
      ['projectsId', 'environmentsId', 'usersId'],
      True
  )
  PROJECTS_AGENT_ENVIRONMENTS_USERS_SESSIONS = (
      'projects.agent.environments.users.sessions',
      'projects/{projectsId}/agent/environments/{environmentsId}/users/'
      '{usersId}/sessions/{sessionsId}',
      {},
      ['projectsId', 'environmentsId', 'usersId', 'sessionsId'],
      True
  )
  PROJECTS_AGENT_ENVIRONMENTS_USERS_SESSIONS_CONTEXTS = (
      'projects.agent.environments.users.sessions.contexts',
      '{+name}',
      {
          '':
              'projects/{projectsId}/agent/environments/{environmentsId}/'
              'users/{usersId}/sessions/{sessionsId}/contexts/{contextsId}',
      },
      ['name'],
      True
  )
  PROJECTS_AGENT_ENVIRONMENTS_USERS_SESSIONS_ENTITYTYPES = (
      'projects.agent.environments.users.sessions.entityTypes',
      '{+name}',
      {
          '':
              'projects/{projectsId}/agent/environments/{environmentsId}/'
              'users/{usersId}/sessions/{sessionsId}/entityTypes/'
              '{entityTypesId}',
      },
      ['name'],
      True
  )
  PROJECTS_AGENT_INTENTS = (
      'projects.agent.intents',
      '{+name}',
      {
          '':
              'projects/{projectsId}/agent/intents/{intentsId}',
      },
      ['name'],
      True
  )
  PROJECTS_AGENT_KNOWLEDGEBASES = (
      'projects.agent.knowledgeBases',
      '{+name}',
      {
          '':
              'projects/{projectsId}/agent/knowledgeBases/{knowledgeBasesId}',
      },
      ['name'],
      True
  )
  PROJECTS_AGENT_KNOWLEDGEBASES_DOCUMENTS = (
      'projects.agent.knowledgeBases.documents',
      '{+name}',
      {
          '':
              'projects/{projectsId}/agent/knowledgeBases/{knowledgeBasesId}/'
              'documents/{documentsId}',
      },
      ['name'],
      True
  )
  PROJECTS_AGENT_SESSIONS = (
      'projects.agent.sessions',
      'projects/{projectsId}/agent/sessions/{sessionsId}',
      {},
      ['projectsId', 'sessionsId'],
      True
  )
  PROJECTS_AGENT_SESSIONS_CONTEXTS = (
      'projects.agent.sessions.contexts',
      '{+name}',
      {
          '':
              'projects/{projectsId}/agent/sessions/{sessionsId}/contexts/'
              '{contextsId}',
      },
      ['name'],
      True
  )
  PROJECTS_AGENT_SESSIONS_ENTITYTYPES = (
      'projects.agent.sessions.entityTypes',
      '{+name}',
      {
          '':
              'projects/{projectsId}/agent/sessions/{sessionsId}/entityTypes/'
              '{entityTypesId}',
      },
      ['name'],
      True
  )
  PROJECTS_AGENT_VERSIONS = (
      'projects.agent.versions',
      '{+name}',
      {
          '':
              'projects/{projectsId}/agent/versions/{versionsId}',
      },
      ['name'],
      True
  )
  PROJECTS_CONVERSATIONDATASETS = (
      'projects.conversationDatasets',
      '{+name}',
      {
          '':
              'projects/{projectsId}/conversationDatasets/'
              '{conversationDatasetsId}',
      },
      ['name'],
      True
  )
  PROJECTS_CONVERSATIONMODELS = (
      'projects.conversationModels',
      '{+name}',
      {
          '':
              'projects/{projectsId}/conversationModels/'
              '{conversationModelsId}',
      },
      ['name'],
      True
  )
  PROJECTS_CONVERSATIONMODELS_EVALUATIONS = (
      'projects.conversationModels.evaluations',
      '{+name}',
      {
          '':
              'projects/{projectsId}/conversationModels/'
              '{conversationModelsId}/evaluations/{evaluationsId}',
      },
      ['name'],
      True
  )
  PROJECTS_CONVERSATIONPROFILES = (
      'projects.conversationProfiles',
      '{+name}',
      {
          '':
              'projects/{projectsId}/conversationProfiles/'
              '{conversationProfilesId}',
      },
      ['name'],
      True
  )
  PROJECTS_CONVERSATIONS = (
      'projects.conversations',
      '{+name}',
      {
          '':
              'projects/{projectsId}/conversations/{conversationsId}',
      },
      ['name'],
      True
  )
  PROJECTS_CONVERSATIONS_PARTICIPANTS = (
      'projects.conversations.participants',
      '{+name}',
      {
          '':
              'projects/{projectsId}/conversations/{conversationsId}/'
              'participants/{participantsId}',
      },
      ['name'],
      True
  )
  PROJECTS_KNOWLEDGEBASES = (
      'projects.knowledgeBases',
      '{+name}',
      {
          '':
              'projects/{projectsId}/knowledgeBases/{knowledgeBasesId}',
      },
      ['name'],
      True
  )
  PROJECTS_KNOWLEDGEBASES_DOCUMENTS = (
      'projects.knowledgeBases.documents',
      '{+name}',
      {
          '':
              'projects/{projectsId}/knowledgeBases/{knowledgeBasesId}/'
              'documents/{documentsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS = (
      'projects.locations',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_AGENT_ENTITYTYPES = (
      'projects.locations.agent.entityTypes',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/agent/'
              'entityTypes/{entityTypesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_AGENT_ENVIRONMENTS = (
      'projects.locations.agent.environments',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/agent/'
              'environments/{environmentsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_AGENT_ENVIRONMENTS_USERS = (
      'projects.locations.agent.environments.users',
      'projects/{projectsId}/locations/{locationsId}/agent/environments/'
      '{environmentsId}/users/{usersId}',
      {},
      ['projectsId', 'locationsId', 'environmentsId', 'usersId'],
      True
  )
  PROJECTS_LOCATIONS_AGENT_ENVIRONMENTS_USERS_SESSIONS = (
      'projects.locations.agent.environments.users.sessions',
      'projects/{projectsId}/locations/{locationsId}/agent/environments/'
      '{environmentsId}/users/{usersId}/sessions/{sessionsId}',
      {},
      ['projectsId', 'locationsId', 'environmentsId', 'usersId', 'sessionsId'],
      True
  )
  PROJECTS_LOCATIONS_AGENT_ENVIRONMENTS_USERS_SESSIONS_CONTEXTS = (
      'projects.locations.agent.environments.users.sessions.contexts',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/agent/'
              'environments/{environmentsId}/users/{usersId}/sessions/'
              '{sessionsId}/contexts/{contextsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_AGENT_ENVIRONMENTS_USERS_SESSIONS_ENTITYTYPES = (
      'projects.locations.agent.environments.users.sessions.entityTypes',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/agent/'
              'environments/{environmentsId}/users/{usersId}/sessions/'
              '{sessionsId}/entityTypes/{entityTypesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_AGENT_INTENTS = (
      'projects.locations.agent.intents',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/agent/intents/'
              '{intentsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_AGENT_SESSIONS = (
      'projects.locations.agent.sessions',
      'projects/{projectsId}/locations/{locationsId}/agent/sessions/'
      '{sessionsId}',
      {},
      ['projectsId', 'locationsId', 'sessionsId'],
      True
  )
  PROJECTS_LOCATIONS_AGENT_SESSIONS_CONTEXTS = (
      'projects.locations.agent.sessions.contexts',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/agent/sessions/'
              '{sessionsId}/contexts/{contextsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_AGENT_SESSIONS_ENTITYTYPES = (
      'projects.locations.agent.sessions.entityTypes',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/agent/sessions/'
              '{sessionsId}/entityTypes/{entityTypesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_AGENT_VERSIONS = (
      'projects.locations.agent.versions',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/agent/versions/'
              '{versionsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_CONVERSATIONDATASETS = (
      'projects.locations.conversationDatasets',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/'
              'conversationDatasets/{conversationDatasetsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_CONVERSATIONMODELS = (
      'projects.locations.conversationModels',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/'
              'conversationModels/{conversationModelsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_CONVERSATIONMODELS_EVALUATIONS = (
      'projects.locations.conversationModels.evaluations',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/'
              'conversationModels/{conversationModelsId}/evaluations/'
              '{evaluationsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_CONVERSATIONPROFILES = (
      'projects.locations.conversationProfiles',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/'
              'conversationProfiles/{conversationProfilesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_CONVERSATIONS = (
      'projects.locations.conversations',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/conversations/'
              '{conversationsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_CONVERSATIONS_PARTICIPANTS = (
      'projects.locations.conversations.participants',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/conversations/'
              '{conversationsId}/participants/{participantsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_KNOWLEDGEBASES = (
      'projects.locations.knowledgeBases',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/knowledgeBases/'
              '{knowledgeBasesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_KNOWLEDGEBASES_DOCUMENTS = (
      'projects.locations.knowledgeBases.documents',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/knowledgeBases/'
              '{knowledgeBasesId}/documents/{documentsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_OPERATIONS = (
      'projects.locations.operations',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/operations/'
              '{operationsId}',
      },
      ['name'],
      True
  )
  PROJECTS_OPERATIONS = (
      'projects.operations',
      '{+name}',
      {
          '':
              'projects/{projectsId}/operations/{operationsId}',
      },
      ['name'],
      True
  )

  def __init__(self, collection_name, path, flat_paths, params,
               enable_uri_parsing):
    self.collection_name = collection_name
    self.path = path
    self.flat_paths = flat_paths
    self.params = params
    self.enable_uri_parsing = enable_uri_parsing



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for deleting access approval settings."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import textwrap

from googlecloudsdk.api_lib.access_approval import settings
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.access_approval import parent


class Update(base.Command):
  """Update Access Approval settings.

  Update the Access Approval settings associated with a project, a folder, or
  organization. Partial updates are supported (for example, you can update the
  notification emails without modifying the enrolled services).
  """

  detailed_help = {
      'EXAMPLES':
          textwrap.dedent("""\
    Update notification emails associated with project 'p1', run:

        $ {command} --project=p1 --notification_emails='foo@example.com, bar@example.com'

    Enable Access Approval enforcement for folder 'f1':

        $ {command} --folder=f1 --enrolled_services=all

    Enable Access Approval enforcement for organization 'org1' for only Cloud Storage and Compute
    products and set the notification emails at the same time:

        $ {command} --organization=org1 --enrolled_services='storage.googleapis.com,compute.googleapis.com' --notification_emails='security_team@example.com'
        """),
  }

  @staticmethod
  def Args(parser):
    """Add command-specific args."""
    parent.Args(parser)
    parser.add_argument(
        '--notification_emails',
        help='Comma-separated list of email addresses to which notifications relating to approval requests should be sent or \'\' to clear all saved notification emails.'
    )
    parser.add_argument(
        '--enrolled_services',
        help='Comma-separated list of services to enroll for Access Approval or \'all\' for all supported services. Note for project and folder enrollments, only \'all\' is supported. Use \'\' to clear all enrolled services.'
    )

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Returns:
      Some value that we want to have printed later.
    """
    p = parent.GetParent(args)

    if args.notification_emails is None and args.enrolled_services is None:
      raise exceptions.MinimumArgumentException(
          ['--notification_emails', '--enrolled_services'],
          'must specify at least one of these flags')

    update_mask = []
    emails_list = []
    if args.notification_emails is not None:
      update_mask.append('notification_emails')
      if args.notification_emails:
        emails_list = args.notification_emails.split(',')
        emails_list = [i.strip() for i in emails_list]

    services_list = []
    if args.enrolled_services is not None:
      update_mask.append('enrolled_services')
      if args.enrolled_services:
        services_list = args.enrolled_services.split(',')
        services_list = [i.strip() for i in services_list]

    return settings.Update(
        name=('%s/accessApprovalSettings' % p),
        notification_emails=emails_list,
        enrolled_services=services_list,
        update_mask=','.join(update_mask))



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.cloud.pubsublite_v1.types import common


__protobuf__ = proto.module(
    package="google.cloud.pubsublite.v1",
    manifest={
        "InitialPublishRequest",
        "InitialPublishResponse",
        "MessagePublishRequest",
        "MessagePublishResponse",
        "PublishRequest",
        "PublishResponse",
    },
)


class InitialPublishRequest(proto.Message):
    r"""The first request that must be sent on a newly-opened stream.

    Attributes:
        topic (str):
            The topic to which messages will be written.
        partition (int):
            The partition within the topic to which messages will be
            written. Partitions are zero indexed, so ``partition`` must
            be in the range [0, topic.num_partitions).
    """

    topic = proto.Field(proto.STRING, number=1,)
    partition = proto.Field(proto.INT64, number=2,)


class InitialPublishResponse(proto.Message):
    r"""Response to an InitialPublishRequest.
    """


class MessagePublishRequest(proto.Message):
    r"""Request to publish messages to the topic.

    Attributes:
        messages (Sequence[google.cloud.pubsublite_v1.types.PubSubMessage]):
            The messages to publish.
    """

    messages = proto.RepeatedField(
        proto.MESSAGE, number=1, message=common.PubSubMessage,
    )


class MessagePublishResponse(proto.Message):
    r"""Response to a MessagePublishRequest.

    Attributes:
        start_cursor (google.cloud.pubsublite_v1.types.Cursor):
            The cursor of the first published message in
            the batch. The cursors for any remaining
            messages in the batch are guaranteed to be
            sequential.
    """

    start_cursor = proto.Field(proto.MESSAGE, number=1, message=common.Cursor,)


class PublishRequest(proto.Message):
    r"""Request sent from the client to the server on a stream.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        initial_request (google.cloud.pubsublite_v1.types.InitialPublishRequest):
            Initial request on the stream.

            This field is a member of `oneof`_ ``request_type``.
        message_publish_request (google.cloud.pubsublite_v1.types.MessagePublishRequest):
            Request to publish messages.

            This field is a member of `oneof`_ ``request_type``.
    """

    initial_request = proto.Field(
        proto.MESSAGE, number=1, oneof="request_type", message="InitialPublishRequest",
    )
    message_publish_request = proto.Field(
        proto.MESSAGE, number=2, oneof="request_type", message="MessagePublishRequest",
    )


class PublishResponse(proto.Message):
    r"""Response to a PublishRequest.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        initial_response (google.cloud.pubsublite_v1.types.InitialPublishResponse):
            Initial response on the stream.

            This field is a member of `oneof`_ ``response_type``.
        message_response (google.cloud.pubsublite_v1.types.MessagePublishResponse):
            Response to publishing messages.

            This field is a member of `oneof`_ ``response_type``.
    """

    initial_response = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="response_type",
        message="InitialPublishResponse",
    )
    message_response = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="response_type",
        message="MessagePublishResponse",
    )


__all__ = tuple(sorted(__protobuf__.manifest))



from typing import Generic, TypeVar, List, Optional
import heapq

_T = TypeVar("_T")


class SortedList(Generic[_T]):
    _vals: List[_T]

    def __init__(self):
        self._vals = []

    def push(self, val: _T):
        heapq.heappush(self._vals, val)

    def peek(self) -> Optional[_T]:
        if self.empty():
            return None
        return self._vals[0]

    def pop(self):
        heapq.heappop(self._vals)

    def empty(self) -> bool:
        return not bool(self._vals)



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Exports a Gcloud Deploy target resource."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import textwrap

from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.deploy import export_util
from googlecloudsdk.command_lib.deploy import manifest_util
from googlecloudsdk.command_lib.deploy import resource_args
from googlecloudsdk.command_lib.deploy import target_util
from googlecloudsdk.command_lib.export import util as core_export_util

_DETAILED_HELP = {
    'DESCRIPTION':
        '{description}',
    'EXAMPLES':
        textwrap.dedent("""\

      To return the .yaml definition of the target 'test-target' in region 'us-central1', run:

        $ {command} test-target --region=us-central1

      """)
}


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA,
                    base.ReleaseTrack.GA)
class Export(base.ExportCommand):
  """Returns the .yaml definition of the specified target.

  The exported YAML definition can be applied by 'deploy apply' command.
  """
  detailed_help = _DETAILED_HELP

  @staticmethod
  def Args(parser):
    resource_args.AddTargetResourceArg(parser, positional=True)
    core_export_util.AddExportFlags(parser)

  def Run(self, args):
    """Entry point of the export command.

    Args:
      args: argparse.Namespace, An object that contains the values for the
        arguments specified in the .Args() method.
    """
    target_ref = args.CONCEPTS.target.Parse()
    target_obj = target_util.GetTarget(target_ref)
    manifest = manifest_util.ProtoToManifest(target_obj, target_ref,
                                             manifest_util.TARGET_KIND_V1BETA1)

    export_util.Export(manifest, args)



# -*- coding: utf-8 -*- #
# Copyright 2014 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Resource keys supplementary help."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import textwrap

from googlecloudsdk.calliope import base


class Keys(base.TopicCommand):
  """Resource keys supplementary help."""

  detailed_help = {

      'DESCRIPTION': textwrap.dedent("""\
          A *resource* is a JSON-serializable object organized as a tree.
          Each node is a scalar, indexed array, or dictionary. Structs and
          classes are represented by dictionaries indexed by member name.

          Each node is reachable by a unique path of names from the root. A
          node *key* is the path names separated by '.'. [_number_] represents
          an array index. For example:

            foo
            foo.bar
            abc.def[3].ghi

          The resource keys and data values for any _gcloud_ _list_
          command can be printed by running _gcloud_ ... _list_
          *--format=flattened*. See the command specific documentation for
          details on specific resource keys.
          """),

      'EXAMPLES': textwrap.dedent("""\
          This command lists the keys and values for the *regions* resource:

            $ gcloud compute regions list --format=flattened

          and here is sample output for the command:

            ---
            creationTimestamp: 2013-05-23T07:02:09.522-07:00
            description:       us-central1
            id:                22115839677829654
            kind:              compute#region
            name:              us-central1
            quotas[0].limit:   24.0
            quotas[0].metric:  CPUS
            quotas[0].usage:   15.0
            quotas[1].limit:   5120.0
            quotas[1].metric:  DISKS_TOTAL_GB
            quotas[1].usage:   1416.0
            quotas[2].limit:   7.0
            quotas[2].metric:  STATIC_ADDRESSES
            quotas[2].usage:   1.0
            quotas[3].limit:   23.0
            quotas[3].metric:  IN_USE_ADDRESSES
            quotas[3].usage:   16.0
            quotas[4].limit:   1024.0
            quotas[4].metric:  SSD_TOTAL_GB
            quotas[4].usage:   0.0
            quotas[5].limit:   1500.0
            quotas[5].metric:  LOCAL_SSD_TOTAL_GB
            quotas[5].usage:   750.0
            selfLink:          https://www.googleapis.com/.../us-central1
            status:            UP
            zones[0]:          us-central1-a
            zones[1]:          us-central1-b
            zones[2]:          us-central1-f

          The *list* command produces a resource list. The keys are to the
          left of ':' and the values are to the right.
          """),
      }



# Lint as: python3
# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Get the root CA certs for all active CAs in a CA Pool."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.privateca import base as privateca_base
from googlecloudsdk.api_lib.privateca import request_utils
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.privateca import pem_utils
from googlecloudsdk.command_lib.privateca import resource_args
from googlecloudsdk.core import log
from googlecloudsdk.core.util import files


@base.ReleaseTracks(base.ReleaseTrack.GA)
class GetCaCerts(base.Command):
  r"""Get the root CA certs for all active CAs in the CA pool.

  ## EXAMPLES

    To get the root CA certs for all active CAs in the CA pool:

      $ {command} my-pool --output-file=ca-certificates.pem \
          --location=us-west1
  """

  @staticmethod
  def Args(parser):
    resource_args.AddCaPoolPositionalResourceArg(
        parser, 'whose CA certificates should be fetched')
    base.Argument(
        '--output-file',
        help='The path where the concatenated PEM certificates will be '
             'written. This will include the root CA certificate for each '
             'active CA in the CA pool. ',
        required=True).AddToParser(parser)

  def _GetRootCerts(self, ca_pool_ref):
    """Returns the root CA certs for all active CAs in the CA pool."""
    client = privateca_base.GetClientInstance('v1')
    messages = privateca_base.GetMessagesModule('v1')

    fetch_ca_certs_response = client.projects_locations_caPools.FetchCaCerts(
        messages.PrivatecaProjectsLocationsCaPoolsFetchCaCertsRequest(
            caPool=ca_pool_ref.RelativeName(),
            fetchCaCertsRequest=messages.FetchCaCertsRequest(
                requestId=request_utils.GenerateRequestId())))
    root_certs = [
        chain.certificates[-1] for chain in fetch_ca_certs_response.caCerts
    ]
    return ''.join(pem_utils.PemChainForOutput(root_certs))

  def Run(self, args):
    ca_pool_ref = args.CONCEPTS.ca_pool.Parse()
    pem_bag = self._GetRootCerts(ca_pool_ref)
    files.WriteFileContents(args.output_file, pem_bag)
    log.status.write('Exported the CA certificates to [{}].'.format(
        args.output_file))



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Flags and helpers for the compute reservations commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.instances import flags as instance_flags


def GetDescriptionFlag():
  return base.Argument(
      '--description',
      help='An optional description of the reservation to create.')


def GetRequireSpecificAllocation():
  help_text = """\
  Indicates whether the reservation can be consumed by VMs with "any reservation"
  defined. If enabled, then only VMs that target this reservation by name using
  `--reservation-affinity=specific` can consume from this reservation.
  """
  return base.Argument(
      '--require-specific-reservation', action='store_true', help=help_text)


def GetVmCountFlag(required=True):
  help_text = """\
  The number of VM instances that are allocated to this reservation.
  The value of this field must be an int in the range [1, 1000].
  """
  return base.Argument(
      '--vm-count', required=required, type=int, help=help_text)


def GetMinCpuPlatform():
  """Gets the --min-cpu-platform flag."""
  return base.Argument(
      '--min-cpu-platform',
      help='Optional minimum CPU platform of the reservation to create.')


def GetLocationHint():
  """Gets the --location-hint flag."""
  return base.Argument(
      '--location-hint',
      hidden=True,
      help="""\
      Used by internal tools to control sub-zone location of the instance.
      """)


def GetMachineType(required=True):
  """Gets the --machine-type flag."""
  help_text = """\
  The type of machine (name only) which has a fixed number of vCPUs and a fixed
  amount of memory. This also includes specifying custom machine type following
  `custom-number_of_CPUs-amount_of_memory` pattern, e.g. `custom-32-29440`.
  """
  return base.Argument('--machine-type', required=required, help=help_text)


def GetLocalSsdFlag(custom_name=None):
  """Gets the --local-ssd flag."""
  help_text = """\
  Manage the size and the interface of local SSD to use. See
  https://cloud.google.com/compute/docs/disks/local-ssd for more information.
  *interface*::: The kind of disk interface exposed to the VM for this SSD. Valid
  values are `scsi` and `nvme`. SCSI is the default and is supported by more
  guest operating systems. NVME may provide higher performance.
  *size*::: The size of the local SSD in base-2 GB.
  """
  return base.Argument(
      custom_name if custom_name else '--local-ssd',
      type=arg_parsers.ArgDict(spec={
          'interface': (lambda x: x.upper()),
          'size': int,
      }),
      action='append',
      help=help_text)


def GetAcceleratorFlag(custom_name=None):
  """Gets the --accelerator flag."""
  help_text = """\
  Manage the configuration of the type and number of accelerator cards attached.
  *count*::: The number of accelerators to attach to each instance in the reservation.
  *type*::: The specific type (e.g. `nvidia-tesla-k80` for nVidia Tesla K80) of
  accelerator to attach to instances in the reservation. Use `gcloud compute accelerator-types list`
  to learn about all available accelerator types.
  """
  return base.Argument(
      custom_name if custom_name else '--accelerator',
      type=arg_parsers.ArgDict(
          spec={
              'count': int,
              'type': str,
          }, required_keys=['count', 'type']),
      action='append',
      help=help_text)


def GetSharedSettingFlag(custom_name=None):
  """Gets the --share-setting flag."""
  help_text = """\
  Specify if this reservation is shared, and if so, the type of sharing. If you
  omit this flag, this value is local (not shared) by default.
  """
  return base.Argument(
      custom_name if custom_name else '--share-setting',
      choices=['local', 'projects', 'folders'],
      help=help_text)


def GetShareWithFlag(custom_name=None):
  """Gets the --share-with flag."""
  help_text = """\
  If this reservation is shared (--share-setting is not local), provide a list
  of all of the specific projects or folders that this reservation is shared
  with. List must contain project IDs or project numbers or folder IDs.
  """
  return base.Argument(
      custom_name if custom_name else '--share-with',
      type=arg_parsers.ArgList(min_length=1),
      metavar='SHARE_WITH',
      help=help_text)


def GetAddShareWithFlag(custom_name=None):
  """Gets the --add-share-with flag."""
  help_text = """\
  A list of specific projects to add to the list of projects that this
  reservation is shared with. List must contain project IDs or project numbers.
  """
  return base.Argument(
      custom_name if custom_name else '--add-share-with',
      type=arg_parsers.ArgList(min_length=1),
      metavar='PROJECT',
      help=help_text)


def GetRemoveShareWithFlag(custom_name=None):
  """Gets the --remove-share-with flag."""
  help_text = """\
  A list of specific projects to remove from the list of projects that this
  reservation is shared with. List must contain project IDs or project numbers.
  """
  return base.Argument(
      custom_name if custom_name else '--remove-share-with',
      type=arg_parsers.ArgList(min_length=1),
      metavar='PROJECT',
      help=help_text)


def AddCreateFlags(parser,
                   support_fleet=False,
                   support_share_setting=False):
  """Adds all flags needed for the create command."""
  GetDescriptionFlag().AddToParser(parser)

  group = base.ArgumentGroup(
      'Manage the specific SKU reservation properties.', required=True)

  group.AddArgument(GetRequireSpecificAllocation())
  group.AddArgument(GetVmCountFlag())
  group.AddArgument(GetMinCpuPlatform())
  group.AddArgument(GetMachineType())
  group.AddArgument(GetLocalSsdFlag())
  group.AddArgument(GetAcceleratorFlag())
  group.AddArgument(GetLocationHint())
  if support_fleet:
    group.AddArgument(instance_flags.AddMaintenanceFreezeDuration())
    group.AddArgument(instance_flags.AddMaintenanceInterval())
  group.AddToParser(parser)

  if support_share_setting:
    share_group = base.ArgumentGroup(
        'Manage the properties of a shared reservation.', required=False)
    share_group.AddArgument(GetSharedSettingFlag())
    share_group.AddArgument(GetShareWithFlag())
    share_group.AddToParser(parser)



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Export Binary Authorization system policy command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.container.binauthz import apis
from googlecloudsdk.api_lib.container.binauthz import system_policy
from googlecloudsdk.api_lib.container.binauthz import util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.container.binauthz import arg_parsers


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class ExportSystemPolicy(base.Command):
  """Export the Binary Authorization system policy.

  For reliability reasons, the system policy is updated one region at a time.
  Because of this precaution, the system policy can differ between regions
  during an update. Use --location to view the system policy of a specific
  region.

  If --location is not specified, an arbitrary region is used. (Specifically, a
  region in the last group of regions to receive updates. Since most changes are
  additions, this will show the minimal set of system images that are allowed
  in all regions.)

  ## EXAMPLES

  To view the system policy:

      $ {command}

  To view the system policy in the region us-central1:

      $ {command} --location=us-central1
  """

  @classmethod
  def Args(cls, parser):
    parser.add_argument(
        '--location',
        # Although the name is "location" for consistency with other gcloud
        # commands, only regions are allowed (not other locations, like zones).
        choices=arg_parsers.BINAUTHZ_ENFORCER_REGIONS,
        required=False,
        default='global',
        help='The region for which to get the system policy (or "global").')

  def Run(self, args):
    api_version = apis.GetApiVersion(self.ReleaseTrack())
    return system_policy.Client(api_version).Get(
        util.GetSystemPolicyRef(args.location))



# -*- coding: utf-8 -*- #
# Copyright 2020 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""`gcloud service-directory endpoints create` command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.service_directory import endpoints
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.service_directory import flags
from googlecloudsdk.command_lib.service_directory import resource_args
from googlecloudsdk.command_lib.service_directory import util
from googlecloudsdk.core import log

_RESOURCE_TYPE = 'endpoint'
_ENDPOINT_LIMIT = 512


@base.ReleaseTracks(base.ReleaseTrack.GA)
class Create(base.CreateCommand):
  """Creates an endpoint."""

  detailed_help = {
      'EXAMPLES':
          """\
          To create a Service Directory endpoint, run:

            $ {command} my-endpoint --service=my-service --namespace=my-namespace --location=us-east1 --address=1.2.3.4 --port=5 --annotations=a=b,c=d  --network=projects/123456789/locations/global/networks/default
          """,
  }

  @staticmethod
  def Args(parser):
    resource_args.AddEndpointResourceArg(
        parser,
        """to create. The endpoint id must be 1-63 characters long and match
        the regular expression `[a-z](?:[-a-z0-9]{0,61}[a-z0-9])?` which means
        the first character must be a lowercase letter, and all following
        characters must be a dash, lowercase letter, or digit, except the last
        character, which cannot be a dash.""")
    flags.AddAddressFlag(parser)
    flags.AddPortFlag(parser)
    flags.AddAnnotationsFlag(parser, _RESOURCE_TYPE, _ENDPOINT_LIMIT)
    flags.AddNetworkFlag(parser)

  def Run(self, args):
    client = endpoints.EndpointsClient()
    endpoint_ref = args.CONCEPTS.endpoint.Parse()
    annotations = util.ParseAnnotationsArg(args.annotations, _RESOURCE_TYPE)

    result = client.Create(endpoint_ref, args.address, args.port, annotations,
                           args.network)
    log.CreatedResource(endpoint_ref.endpointsId, _RESOURCE_TYPE)

    return result


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
class CreateBeta(base.CreateCommand):
  """Creates an endpoint."""

  detailed_help = {
      'EXAMPLES':
          """\
          To create a Service Directory endpoint, run:

            $ {command} my-endpoint --service=my-service --namespace=my-namespace --location=us-east1 --address=1.2.3.4 --port=5 --metadata=a=b,c=d --network=projects/123456789/locations/global/networks/default
          """,
  }

  @staticmethod
  def Args(parser):
    resource_args.AddEndpointResourceArg(
        parser,
        """to create. The endpoint id must be 1-63 characters long and match
        the regular expression `[a-z](?:[-a-z0-9]{0,61}[a-z0-9])?` which means
        the first character must be a lowercase letter, and all following
        characters must be a dash, lowercase letter, or digit, except the last
        character, which cannot be a dash.""")
    flags.AddAddressFlag(parser)
    flags.AddPortFlag(parser)
    flags.AddMetadataFlag(parser, _RESOURCE_TYPE, _ENDPOINT_LIMIT)
    flags.AddNetworkFlag(parser)

  def Run(self, args):
    client = endpoints.EndpointsClientBeta()
    endpoint_ref = args.CONCEPTS.endpoint.Parse()
    metadata = util.ParseMetadataArg(args.metadata, _RESOURCE_TYPE)

    result = client.Create(endpoint_ref, args.address, args.port, metadata,
                           args.network)
    log.CreatedResource(endpoint_ref.endpointsId, _RESOURCE_TYPE)

    return result



# -*- coding: utf-8 -*- #
# Copyright 2019 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to update a Data Fusion instance."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from apitools.base.py import encoding

from googlecloudsdk.api_lib.data_fusion import datafusion as df
from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.data_fusion import operation_poller
from googlecloudsdk.command_lib.data_fusion import resource_args
from googlecloudsdk.command_lib.util.args import labels_util
from googlecloudsdk.core import log


class Update(base.UpdateCommand):
  # pylint:disable=line-too-long
  r"""Updates a Cloud Data Fusion instance."""
  detailed_help = {
      'DESCRIPTION':
          """\
       If run asynchronously with `--async`, exits after printing an operation
       that can be used to poll the status of the creation operation via:

         {command} operations list
          """,
      'EXAMPLES':
          """\
        To update instance 'my-instance' in project 'my-project' and location
        'my-location' to version `6.1.0.0`, run:

          $ {command} --project=my-project --location=my-location --version=6.1.0.0 my-instance
          """,
  }

  # REST API Field Names for the updateMask
  FIELD_PATH_OPTIONS = 'options'
  FIELD_PATH_ENABLE_RBAC = 'enableRbac'
  FIELD_PATH_ENABLE_STACKDRIVER_LOGGING = 'enableStackdriverLogging'
  FIELD_PATH_ENABLE_STACKDRIVER_MONITORING = 'enableStackdriverMonitoring'

  @staticmethod
  def Args(parser):
    resource_args.AddInstanceResourceArg(parser, 'Instance to update.')
    base.ASYNC_FLAG.AddToParser(parser)
    labels_util.AddCreateLabelsFlags(parser)
    parser.add_argument(
        '--enable_stackdriver_logging',
        action='store_true',
        help='Enable Stackdriver logging for this Data Fusion instance.')
    parser.add_argument(
        '--enable_stackdriver_monitoring',
        action='store_true',
        help='Enable Stackdriver monitoring for this Data Fusion instance.')
    parser.add_argument(
        '--enable_rbac',
        action='store_true',
        help='Enable granular role-based access control for this Data Fusion instance.'
    )
    parser.add_argument(
        '--options',
        type=arg_parsers.ArgDict(),
        metavar='KEY=VALUE',
        help='Options to use for instance update, '
        'specified as KEY1=VALUE1,KEY2=VALUE2.')
    parser.add_argument('--version', help='Version of Datafusion to update to.')

  def Run(self, args):
    datafusion = df.Datafusion()
    instance_ref = args.CONCEPTS.instance.Parse()

    labels = args.labels or {}
    enable_stackdriver_logging = None
    enable_stackdriver_monitoring = None
    enable_rbac = None
    options = {}

    fields_to_update = []
    if args.IsSpecified('options'):
      options = args.options
      fields_to_update.append(self.FIELD_PATH_OPTIONS)
    if args.IsSpecified('enable_rbac'):
      fields_to_update.append(self.FIELD_PATH_ENABLE_RBAC)
      enable_rbac = args.enable_rbac
    if args.IsSpecified('enable_stackdriver_logging'):
      fields_to_update.append(self.FIELD_PATH_ENABLE_STACKDRIVER_LOGGING)
      enable_stackdriver_logging = args.enable_stackdriver_logging
    if args.IsSpecified('enable_stackdriver_monitoring'):
      fields_to_update.append(self.FIELD_PATH_ENABLE_STACKDRIVER_MONITORING)
      enable_stackdriver_monitoring = args.enable_stackdriver_monitoring

    version = args.version
    instance = datafusion.messages.Instance(
        name=instance_ref.RelativeName(),
        version=version,
        enableStackdriverLogging=enable_stackdriver_logging,
        enableStackdriverMonitoring=enable_stackdriver_monitoring,
        enableRbac=enable_rbac,
        options=encoding.DictToAdditionalPropertyMessage(
            options, datafusion.messages.Instance.OptionsValue, True),
        labels=encoding.DictToAdditionalPropertyMessage(
            labels, datafusion.messages.Instance.LabelsValue, True))
    request = datafusion.messages.DatafusionProjectsLocationsInstancesPatchRequest(
        instance=instance,
        updateMask=','.join(fields_to_update),
        name=instance_ref.RelativeName())

    operation = datafusion.client.projects_locations_instances.Patch(request)

    if args.async_:
      log.CreatedResource(
          instance_ref.RelativeName(), kind='instance', is_async=True)
      return operation
    else:
      waiter.WaitFor(
          operation_poller.OperationPoller(),
          operation.name,
          'Waiting for [{}] to complete. This may take several minutes.'.format(
              operation.name),
          wait_ceiling_ms=df.OPERATION_TIMEOUT)
      log.UpdatedResource(
          instance_ref.RelativeName(), kind='instance', is_async=False)
